
#include <QtGui>
#include <QDirModel>
#include <QtSql>
#include <QtitanGrid.h>

#include "window.h"


Window::Window()
: DemoMainWindow("QtitanDataGrid")
{
    setWindowTitle(tr("Qtitan grid demo (Adding new row to the grid possibility)"));
    setGeometry(150, 150, 1000, 700);

    m_grid = new Qtitan::Grid();

    QString prefix = QApplication::applicationDirPath();
    prefix += "/..";

    QSqlError err;
    QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE", QString("database_demo"));

    db.setDatabaseName(prefix + "/demos/data/database.sqlite");
    db.setHostName("");
    db.setPort(-1);
    if (!db.open("", ""))
    {
        err = db.lastError();
        QSqlDatabase::removeDatabase(QString("database_demo"));
        QMessageBox::critical(0, "Demo Error", QString("Error: Can't open database - %1, error - %2").arg(prefix + "/demos/data/database.sqlite").arg(err.text()));
        QApplication::exit(1);
        return;
    }

    QSqlTableModel* model = new QSqlTableModel(m_grid, db);
    model->setTable("data");
    model->select();
    if (model->lastError().type() != QSqlError::NoError)
    {
        QMessageBox::critical(0, "Demo Error", "Error: SQL data base is not valid.");
        QApplication::exit(1);
        return;
    }
    model->setEditStrategy(QSqlTableModel::OnFieldChange);

    // Configure grid view
    m_grid->setViewType(Qtitan::Grid::BandedTableView);
    Qtitan::GridBandedTableView* view = m_grid->view<Qtitan::GridBandedTableView>();
    view->options().setBandsHeader(false);
    //view->options().setColumnsHeader(false);
    //view->options().setGroupsHeader(false);
    view->options().setColumnAutoWidth(true);
    view->options().setRowAutoHeight(true);
    //view->options().setBorderWidth(0);
    view->options().setNewRowPlace(Qtitan::NewRowTop);
    Qtitan::GridTableBand* band = view->addBand("Characteristics");
    band = view->addBand("Engineering");
    view->setModel(model);

    Qtitan::GridBandedTableColumn* column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Photo");
    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Registration");
    //column->setRowIndex(0);
    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Aircraft");
    //column->setRowIndex(0);
    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Location");
    column->setRowIndex(1);
    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Date");
    column->setRowIndex(2);

    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Photo");
    column->setEditorType(Qtitan::PictureEditorType);
    Qtitan::GridPictureEditorRepository* pictureEditor = (Qtitan::GridPictureEditorRepository *)column->editorRepository();
    Q_UNUSED(pictureEditor);
    column->setBandIndex(1);
    column->setRowSpan(3);

    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("History");
    column->setEditorType(Qtitan::MemoEditorType);
    column->setBandIndex(1);
    column->setRowSpan(3);

    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Info");
    column->setEditorType(Qtitan::MemoEditorType);
    column->setBandIndex(1);
    column->setRowSpan(3);

    QWidget* settings = new QWidget(this);
    QGridLayout* l = new QGridLayout(settings);

    m_placeLabel = new QLabel(settings);
    m_placeLabel->setText(tr("New row panel place:"));
    m_newRowPosition = new QComboBox(settings);
    m_newRowPosition->addItem("Top");
    m_newRowPosition->addItem("Bottom");
    m_newRowPosition->addItem("Hide");
    connect(m_newRowPosition, SIGNAL(activated(int)), this, SLOT(newRowPositionActivated(int)));
    m_placeLabel->setBuddy(m_newRowPosition);

    m_effectLabel = new QLabel(settings);
    m_effectLabel->setText(tr("Highlight effect:"));
    m_newRowEffect = new QComboBox(settings);
    m_newRowEffect->addItem("Flash");
    m_newRowEffect->addItem("Alpha");
    connect(m_newRowEffect, SIGNAL(activated(int)), this, SLOT(newRowEffectActivated(int)));
    m_effectLabel->setBuddy(m_newRowEffect);

    l->addWidget(m_placeLabel, 0, 0);
    l->addWidget(m_newRowPosition, 0, 1);
    l->addWidget(m_effectLabel, 1, 0);
    l->addWidget(m_newRowEffect, 1, 1);

    m_fastScrollCheck = new QCheckBox(settings);
    m_fastScrollCheck->setText("Fast scroll effect (new)");
    connect(m_fastScrollCheck, SIGNAL(stateChanged(int)), this, SLOT(fastScrollChanged(int)));
    l->addWidget(m_fastScrollCheck, 2, 0, 1, 2);
    m_fastScrollCheck->setChecked(true);

    setDemoWidget(m_grid, settings);
}

void Window::newRowPositionActivated(int index)
{
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();

    switch (index)
    {
    case 0:
        view->options().setNewRowPlace(Qtitan::NewRowTop);
        break;
    case 1:
        view->options().setNewRowPlace(Qtitan::NewRowBottom);
        break;
    default:
        view->options().setNewRowPlace(Qtitan::NewRowNone);
    }
}

void Window::newRowEffectActivated(int index)
{
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();

    switch (index)
    {
    case 0:
        view->options().setNewRowHighlightEffect(Qtitan::RowFlashEffect);
        break;
    case 1:
        view->options().setNewRowHighlightEffect(Qtitan::RowAlphaEffect);
        break;
    default:
        view->options().setNewRowHighlightEffect(Qtitan::RowAlphaEffect);
    }
}

void Window::fastScrollChanged(int state)
{
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();
    view->options().setFastScrollEffect(state == Qt::Checked);
}

