
#include <QtGui>
#include <QDirModel>
#include <QtitanGrid.h>

#include "window.h"


Window::Window()
: DemoMainWindow("QtitanDataGrid")
{
    setWindowTitle(tr("Qtitan Column Editor Demo (Banded table view)"));
    setGeometry(150, 100, 900, 800);

    m_grid = new Qtitan::Grid();
    ColumnEditorDemoModel* model = new ColumnEditorDemoModel(m_grid);

    // Configure grid view
    m_grid->setViewType(Qtitan::Grid::BandedTableView);
    Qtitan::GridBandedTableView* view = m_grid->view<Qtitan::GridBandedTableView>();
    view->options().setBandHeight(300);
    view->options().setModelDecoration(true);

    Qtitan::GridTableBand* engineeringBand = view->addBand("Engineering");
    Qtitan::GridTableBand* characteristics1 = view->addBand("Characteristics 1");
    Qtitan::GridTableBand* characteristics2 = view->addBand("Characteristics 2");
    Qtitan::GridTableBand* characteristics3 = view->addBand("Characteristics 3");
    GridChartColumnEditor* chartEditor = new GridChartColumnEditor();
    characteristics1->setColumnEditor(chartEditor);
    characteristics2->setColumnEditor(chartEditor);
    characteristics3->setColumnEditor(chartEditor);
    engineeringBand->setFixedKind(Qtitan::FixedLeft);
    view->setModel(model);

    Qtitan::GridBandedTableColumn* column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Column 0");
    column->setBandIndex(engineeringBand->index());
    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Column 1");
    column->setBandIndex(engineeringBand->index());
    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Column 2");
    column->setBandIndex(engineeringBand->index());

    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Column 3");
    column->setBandIndex(characteristics1->index());
    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Column 4");
    column->setBandIndex(characteristics1->index());
    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Column 5");
    column->setBandIndex(characteristics1->index());

    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Column 6");
    column->setBandIndex(characteristics2->index());
    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Column 7");
    column->setBandIndex(characteristics2->index());
    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Column 8");
    column->setBandIndex(characteristics2->index());

    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Column 9");
    column->setBandIndex(characteristics3->index());
    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Column 10");
    column->setBandIndex(characteristics3->index());
    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Column 11");
    column->setBandIndex(characteristics3->index());

    //Create settings widget
    QWidget* settings = new QWidget(this);
    QVBoxLayout* l = new QVBoxLayout(settings);
    m_autoWidthCheck = new QCheckBox(settings);
    m_autoWidthCheck->setText("Column auto width");
    connect(m_autoWidthCheck, SIGNAL(stateChanged(int)), this, SLOT(autoWidthStateChanged(int)));
    l->addWidget(m_autoWidthCheck);
    m_autoWidthCheck->setChecked(true);

    m_fastScrollCheck = new QCheckBox(settings);
    m_fastScrollCheck->setText("Fast scroll effect (new)");
    connect(m_fastScrollCheck, SIGNAL(stateChanged(int)), this, SLOT(fastScrollChanged(int)));
    l->addWidget(m_fastScrollCheck);
    m_fastScrollCheck->setChecked(true);

    setDemoWidget(m_grid, settings);
}

void Window::autoWidthStateChanged(int state)
{
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();
    view->options().setColumnAutoWidth(state == Qt::Checked);
}

void Window::fastScrollChanged(int state)
{
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();
    view->options().setFastScrollEffect(state == Qt::Checked);
}

/* GridChartColumnEditor */
GridChartColumnEditor::GridChartColumnEditor()
: Qtitan::GridTableColumnEditor(), m_model(0), m_selectionModel(0)
{
    m_model = new QStandardItemModel(8, 2, this);
    m_model->setHeaderData(0, Qt::Horizontal, tr("Label"));
    m_model->setHeaderData(1, Qt::Horizontal, tr("Quantity"));
    m_selectionModel = new QItemSelectionModel(m_model);
}

QWidget* GridChartColumnEditor::createEditorWidget(GridTableColumnBase* column)
{
    Q_UNUSED(column);
    QAbstractItemView* pieChart = new PieView();
    pieChart->setFrameShape(QFrame::NoFrame);
    pieChart->setModel(m_model);
    loadModel(":/Charts/qtdata.cht");
    pieChart->setSelectionModel(m_selectionModel);
    return pieChart;
}

void GridChartColumnEditor::loadModel(const QString &fileName)
{
    if (!fileName.isEmpty())
    {
        QFile file(fileName);

        if (file.open(QFile::ReadOnly | QFile::Text))
        {
            QTextStream stream(&file);
            QString line;

            m_model->removeRows(0, m_model->rowCount(QModelIndex()), QModelIndex());

            int row = 0;
            do {
                line = stream.readLine();
                if (!line.isEmpty()) {

                    m_model->insertRows(row, 1, QModelIndex());

                    QStringList pieces = line.split(",", QString::SkipEmptyParts);
                    m_model->setData(m_model->index(row, 0, QModelIndex()),
                                   pieces.value(0));
                    m_model->setData(m_model->index(row, 1, QModelIndex()),
                                   pieces.value(1));
                    m_model->setData(m_model->index(row, 0, QModelIndex()),
                                   QColor(pieces.value(2)), Qt::DecorationRole);
                    row++;
                }
            } while (!line.isEmpty());

            file.close();
        }
    }
}


/* ColumnEditorDemoModel */
ColumnEditorDemoModel::ColumnEditorDemoModel(QObject *parent) 
: QAbstractItemModel(parent)
{
} 

QVariant ColumnEditorDemoModel::headerData(int section, Qt::Orientation orientation,
                                      int role) const
{
    Q_UNUSED(orientation);
    Q_UNUSED(role);
    return QString("Column %1").arg(section);
}

QModelIndex ColumnEditorDemoModel::parent(const QModelIndex & /*child*/) const 
{
    return QModelIndex();
} 

bool ColumnEditorDemoModel::hasChildren(const QModelIndex &parent) const 
{  
    if (parent.model() == this || !parent.isValid()) 
    {
        return rowCount(parent) > 0 && columnCount(parent) > 0;
    }
    return false;
} 

int ColumnEditorDemoModel::rowCount(const QModelIndex &parent) const 
{
    if (parent.isValid())
        return 0;
    return 1000;
} 

int ColumnEditorDemoModel::columnCount(const QModelIndex &parent) const 
{
    if (parent.isValid())
        return 0;
    return 12;
}

QModelIndex ColumnEditorDemoModel::index(int row, int column, const QModelIndex &parent) const 
{
    if (parent.isValid())                                   
        return QModelIndex();

    if (row < 0 || row >= rowCount(parent))
        return QModelIndex();

    if (column < 0 || column >= columnCount(parent)) 
        return QModelIndex();

    return createIndex(row, column, (void*)NULL);
}

QVariant ColumnEditorDemoModel::data(const QModelIndex &index, int role) const 
{  
    if (!index.isValid())
        return QVariant();
    
    if (index.row() < 0 || index.row() >= rowCount(index.parent()))
        return QVariant();
    
    if (index.column() < 0 || index.column() >= columnCount(index.parent()))
        return QVariant();
    
    if (role == Qt::DisplayRole || role == Qt::EditRole)        
    {    
        return index.row();
    }

    if (role == Qt::DecorationRole)
    {
        if (index.column() == 5 && index.row() >= 3 && index.row() < 10)
        {
            return QColor(Qt::red);
        }
        if (index.column() == 7 && index.row() >= 5 && index.row() < 15)
        {
            return QColor(Qt::gray);
        }
    }
    return QVariant();
}

Qt::ItemFlags ColumnEditorDemoModel::flags(const QModelIndex &index) const 
{  
    if (!index.isValid())
        return false;
    return Qt::ItemIsEditable; 
}                
