
#include <QtGui>
#include <QDirModel>
#include <QtSql>
#include <QtitanGrid.h>

#include "window.h"


Window::Window()
: DemoMainWindow("QtitanDataGrid")
{
    setWindowTitle(tr("Qtitan grid demo (Fixed columns)"));
    setGeometry(50, 50, 800, 500);

    m_grid = new Qtitan::Grid(this);

    QString prefix = QApplication::applicationDirPath();
    prefix += "/..";

    QSqlError err;
    QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE", QString("database_demo"));

    db.setDatabaseName(prefix + "/demos/data/database.sqlite");
    db.setHostName("");
    db.setPort(-1);
    if (!db.open("", ""))
    {
        err = db.lastError();
        QSqlDatabase::removeDatabase(QString("database_demo"));
        QMessageBox::critical(0, "Demo Error", QString("Error: Can't open database - %1, error - %2").arg(prefix + "/demos/data/database.sqlite").arg(err.text()));
        QApplication::exit(1);
        return;
    }

    QSqlTableModel* model = new QSqlTableModel(m_grid, db);
    model->setTable("data");
    model->select();
    if (model->lastError().type() != QSqlError::NoError)
    {
        QMessageBox::critical(0, "Demo Error", "Error: SQL data base is not valid.");
        QApplication::exit(1);
        return;
    }

    // Configure grid view
    m_grid->setViewType(Qtitan::Grid::TableView);
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();
    view->modelController()->setCacheUsage(Qtitan::UseCache);
    view->options().setGridLineWidth(1);
    view->setModel(model);

    Qtitan::GridTableColumn* column = (Qtitan::GridTableColumn *)view->getColumnByModelColumnName("Photo");
    column->setEditorType(Qtitan::ComboBoxPictureEditorType);
    column->setFixedKind(Qtitan::FixedLeft);

    column = (Qtitan::GridTableColumn *)view->getColumnByModelColumnName("History");
    column->setEditorType(Qtitan::ComboBoxBlobEditorType);
    column->setFixedKind(Qtitan::FixedRight);

    column = (Qtitan::GridTableColumn *)view->getColumnByModelColumnName("Info");
    column->setEditorType(Qtitan::ComboBoxBlobEditorType);
    column->setFixedKind(Qtitan::FixedRight);

    m_autoWidthCheck = new QCheckBox(this);
    m_autoWidthCheck->setText("Column auto width");
    connect(m_autoWidthCheck, SIGNAL(stateChanged(int)), this, SLOT(autoWidthStateChanged(int)));

    setDemoWidget(m_grid, m_autoWidthCheck);
}

void Window::autoWidthStateChanged(int state)
{
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();
    view->options().setColumnAutoWidth(state == Qt::Checked);
}
