
#include <QtGui>
#include <QDirModel>
#include <QtitanGrid.h>

#include "window.h"

/* Window */
Window::Window()
: DemoMainWindow("QtitanDataGrid")
{
    setWindowTitle(tr("Qtitan grid demo (Large data set - 100.000 rows)"));
    setGeometry(50, 50, 800, 500);

    m_grid = new Qtitan::Grid();
    LargeDataSetModel* model = new LargeDataSetModel(m_grid);

    // Configure grid view
    m_grid->setViewType(Qtitan::Grid::TableView);
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();
    view->options().setGridLineWidth(1);
    view->setModel(model);

    Qtitan::GridTableColumn* column = (Qtitan::GridTableColumn *)view->getColumn(0);
    column->setEditorType(Qtitan::CheckBoxEditorType);
    column = (Qtitan::GridTableColumn *)view->getColumn(1);
    column->setEditorType(Qtitan::NumericEditorType);
    ((Qtitan::GridNumericEditorRepository *)column->editorRepository())->setMinimum(-10000);
    ((Qtitan::GridNumericEditorRepository *)column->editorRepository())->setMaximum(10000);
    column = (Qtitan::GridTableColumn *)view->getColumn(2);
    column->setEditorType(Qtitan::StringEditorType);
    column = (Qtitan::GridTableColumn *)view->getColumn(3);
    column->setEditorType(Qtitan::DateEditorType);
    column = (Qtitan::GridTableColumn *)view->getColumn(4);
    column->setEditorType(Qtitan::TimeEditorType);

    //Create settings widget
    QWidget* settings = new QWidget(this);
    QVBoxLayout* l = new QVBoxLayout(settings);
    m_autoWidthCheck = new QCheckBox(settings);
    m_autoWidthCheck->setText("Column auto width");
    connect(m_autoWidthCheck, SIGNAL(stateChanged(int)), this, SLOT(autoWidthStateChanged(int)));
    m_autoWidthCheck->setChecked(true);
    l->addWidget(m_autoWidthCheck);
    m_fastScrollCheck = new QCheckBox(settings);
    m_fastScrollCheck->setText("Fast scroll effect (new)");
    connect(m_fastScrollCheck, SIGNAL(stateChanged(int)), this, SLOT(fastScrollChanged(int)));
    m_fastScrollCheck->setChecked(true);
    l->addWidget(m_fastScrollCheck);
    setDemoWidget(m_grid, settings);
}

void Window::autoWidthStateChanged(int state)
{
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();
    view->options().setColumnAutoWidth(state == Qt::Checked);
}

void Window::fastScrollChanged(int state)
{
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();
    view->options().setFastScrollEffect(state == Qt::Checked);
}

/* LargeDataSetModel */
LargeDataSetModel::LargeDataSetModel(QObject *parent) 
: QAbstractItemModel(parent)
{
    m_values.resize(100000);
    int y = 2009;
    int m = 1;
    int d = 1;
    for (int i = 0; i < m_values.size(); ++i)
    {
        m_values[i].v0 = true;
        m_values[i].v1 = i;
        m_values[i].v2 = QString("String = %1").arg(i);
        m_values[i].v4 = QTime(12, 0, 0);
        if (d > 28)
        {
            d = 1;
            m++;
            if (m > 12)
            {
                m = 1;
                y++;
            }
        }
        m_values[i].v3 = QDate(y, m, d);
        if (!(i % 10))
            d++;
    }
} 

LargeDataSetModel::~LargeDataSetModel()
{
} 

QVariant LargeDataSetModel::headerData(int section, Qt::Orientation orientation,
                                      int role) const
{
    Q_UNUSED(orientation);
    Q_UNUSED(role);
    switch (section)
    {
    case 0:
        {
            return QString("Boolean");
        }
        break;
    case 1:
        {
            return QString("Integer");
        }
        break;
    case 2:
        {
            return QString("String");
        }
        break;
    case 3:
        {
            return QString("Date");
        }
        break;
    case 4:
        {
            return QString("Time");
        }
        break;
    }
    return QVariant();
}

QModelIndex LargeDataSetModel::parent(const QModelIndex & /*child*/) const 
{
    return QModelIndex();
} 

bool LargeDataSetModel::hasChildren(const QModelIndex &parent) const 
{  
    if (parent.model() == this || !parent.isValid()) 
    {
        return rowCount(parent) > 0 && columnCount(parent) > 0;
    }
    return false;
} 

int LargeDataSetModel::rowCount(const QModelIndex &parent) const 
{
    if (parent.isValid())
        return 0;
    return m_values.size();
} 

int LargeDataSetModel::columnCount(const QModelIndex &parent) const 
{
    if (parent.isValid())
        return 0;
    return 5;
}

QModelIndex LargeDataSetModel::index(int row, int column, const QModelIndex &parent) const 
{
    if (parent.isValid())                                   
        return QModelIndex();

    if (row < 0 || row >= rowCount(parent))
        return QModelIndex();

    if (column < 0 || column >= columnCount(parent)) 
        return QModelIndex();

    return createIndex(row, column, (void*)NULL);
}

QVariant LargeDataSetModel::data(const QModelIndex &index, int role) const 
{  
    if (!index.isValid())
        return QVariant();
    
    if (index.row() < 0 || index.row() >= rowCount(index.parent()))
        return QVariant();
    
    if (index.column() < 0 || index.column() >= columnCount(index.parent()))
        return QVariant();
    
    if (role == Qt::DisplayRole || role == Qt::EditRole)        
    {    
        switch (index.column())
        {
        case 0:
            {
                return m_values[index.row()].v0;
            }
            break;
        case 1:
            {
                return m_values[index.row()].v1;
            }
            break;
        case 2:
            {
                return m_values[index.row()].v2;
            }
            break;
        case 3:
            {
                return m_values[index.row()].v3;
            }
            break;
        case 4:
            {
                return m_values[index.row()].v4;
            }
        }
    }
    else if (role == Qt::CheckStateRole)
    {
        switch (index.column())
        {
        case 0:
            {
                return m_values[index.row()].v0;
            }
            break;
        }
    }
    return QVariant();
}

bool LargeDataSetModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    if (!index.isValid())
        return false;
    
    if (index.row() < 0 || index.row() >= rowCount(index.parent()))
        return false;
    
    if (index.column() < 0 || index.column() >= columnCount(index.parent()))
        return false;

    if (role != Qt::EditRole)
        return false;

    switch (index.column())
    {
    case 0:
        {
            m_values[index.row()].v0 = value.toBool();
        }
        break;
    case 1:
        {
            m_values[index.row()].v1 = value.toInt();
        }
        break;
    case 2:
        {
            m_values[index.row()].v2 = value.toString();
        }
        break;
    case 3:
        {
            m_values[index.row()].v3 = value.toDate();
        }
        break;
    case 4:
        {
            m_values[index.row()].v4 = value.toTime();
        }
        break;
    }

    emit dataChanged(index, index);
    return true;
}

Qt::ItemFlags LargeDataSetModel::flags(const QModelIndex &index) const 
{  
    if (!index.isValid())
        return false;
    return Qt::ItemIsEnabled | Qt::ItemIsEditable;
}                

