
#include <QtGui>
#include <QDirModel>
#include <QtitanGrid.h>

#include "window.h"

void addMail(QAbstractItemModel* model, const QString& subject,
             const QString& from, const QDateTime &date, bool attachment, bool read, int status)
{
    model->insertRow(0);
    QPixmap pm;

    model->setData(model->index(0, 0), pm);
    if (attachment)
    {
        pm = QPixmap(":res/attachmentcell.png");
        pm.setMask(pm.createMaskFromColor(Qt::magenta));
    }
    model->setData(model->index(0, 1), pm);
    pm = QPixmap();
    model->setData(model->index(0, 2), pm);

    if (read)
    {
        pm = QPixmap(":res/mail.png");
        pm.setMask(pm.createMaskFromColor(Qt::magenta));
    }
    else
    {
        pm = QPixmap(":res/mailandarrow.png");
        pm.setMask(pm.createMaskFromColor(Qt::magenta));
    }
    model->setData(model->index(0, 3), pm);

    model->setData(model->index(0, 4), date);
    model->setData(model->index(0, 5), subject);
    model->setData(model->index(0, 6), from);

    if (status == 0)
    {
        pm = QPixmap(":res/flag.png");
        pm.setMask(pm.createMaskFromColor(Qt::magenta));
    }
    else
    {
        pm = QPixmap(":res/flag_red.png");
        pm.setMask(pm.createMaskFromColor(Qt::magenta));
    }

    model->setData(model->index(0, 7), pm);
}

QAbstractItemModel *createMailModel(QObject *parent)
{
    QStandardItemModel *model = new QStandardItemModel(0, 8, parent);

    model->setHeaderData(0, Qt::Horizontal, QObject::tr("Importance"));
    model->setHeaderData(1, Qt::Horizontal, QObject::tr("Attachment"));
    model->setHeaderData(2, Qt::Horizontal, QObject::tr("Reminder"));
    model->setHeaderData(3, Qt::Horizontal, QObject::tr("Read"));
    model->setHeaderData(4, Qt::Horizontal, QObject::tr("Created"));
    model->setHeaderData(5, Qt::Horizontal, QObject::tr("Subject"));
    model->setHeaderData(6, Qt::Horizontal, QObject::tr("From"));
    model->setHeaderData(7, Qt::Horizontal, QObject::tr("Status"));



    addMail(model, "Happy New Year!", "Grace K. <grace@software-inc.com>",
            QDateTime(QDate(2006, 12, 31), QTime(17, 03)), false, true, 0);
    addMail(model, "Radically new concept", "Grace K. <grace@software-inc.com>",
            QDateTime(QDate(2006, 12, 22), QTime(9, 44)), true, true, 0);
    addMail(model, "Accounts", "pascale@nospam.com",
            QDateTime(QDate(2006, 12, 31), QTime(12, 50)), false, true, 0);
    addMail(model, "Expenses", "Joe Bloggs <joe@bloggs.com>",
            QDateTime(QDate(2006, 12, 25), QTime(11, 39)), false, false, 0);
    addMail(model, "Re: Expenses", "Andy <andy@nospam.com>",
            QDateTime(QDate(2007, 01, 02), QTime(16, 05)), false, true, 1);
    addMail(model, "Re: Accounts", "Joe Bloggs <joe@bloggs.com>",
            QDateTime(QDate(2007, 01, 03), QTime(14, 18)), false, false, 0);
    addMail(model, "Re: Accounts", "Andy <andy@nospam.com>",
            QDateTime(QDate(2007, 01, 03), QTime(14, 26)), true, false, 0);
    addMail(model, "Sports", "Linda Smith <linda.smith@nospam.com>",
            QDateTime(QDate(2007, 01, 05), QTime(11, 33)), false, true, 1);
    addMail(model, "AW: Sports", "Rolf Newschweinstein <rolfn@nospam.com>",
            QDateTime(QDate(2007, 01, 05), QTime(12, 00)), true, true, 0);
    addMail(model, "RE: Sports", "Petra Schmidt <petras@nospam.com>",
            QDateTime(QDate(2007, 01, 05), QTime(12, 01)), false, true, 0);

    return model;
}

Window::Window()
: DemoMainWindow("QtitanDataGrid")
{
    setWindowTitle(tr("Qtitan MS-Office row grouping demo"));
    setGeometry(50, 50, 600, 300);

    m_grid = new Qtitan::Grid();
    QAbstractItemModel* model = createMailModel(m_grid);

    // Configure grid view
    m_grid->setViewType(Qtitan::Grid::TableView);
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();
    view->options().setModelDecoration(false);
    view->options().setGroupRowStyle(GridGroupOffice11);

    view->setModel(model);

    QPixmap pm;
    pm = QPixmap(":res/attachment.png");
    pm.setMask(pm.createMaskFromColor(Qt::magenta));
    Qtitan::GridTableColumn* column = (Qtitan::GridTableColumn *)view->getColumnByModelColumnName("Attachment");
    column->setIcon(pm);
    column->setIconApperance(Qtitan::IconOnly | Qtitan::IconAlignLeft);
    column->setWidth(pm.width());
    column->setTextAlignment(Qt::AlignCenter);
    column->setHorSizingEnabled(false);
    column->setEditorType(Qtitan::PictureEditorType);
    column->editorRepository()->setEditable(false);
    column->editorRepository()->setAlignment(Qt::AlignCenter);
    ((GridPictureEditorRepository *)column->editorRepository())->setScaled(false);

    pm = QPixmap(":res/importance.png");
    pm.setMask(pm.createMaskFromColor(Qt::magenta));
    column = (Qtitan::GridTableColumn *)view->getColumnByModelColumnName("Importance");
    column->setIcon(pm);
    column->setIconApperance(Qtitan::IconOnly | Qtitan::IconAlignLeft);
    column->setWidth(pm.width());
    column->setTextAlignment(Qt::AlignCenter);
    column->setHorSizingEnabled(false);
    column->setEditorType(Qtitan::PictureEditorType);
    column->editorRepository()->setEditable(false);
    column->editorRepository()->setAlignment(Qt::AlignCenter);
    ((GridPictureEditorRepository *)column->editorRepository())->setScaled(false);

    pm = QPixmap(":res/reminder.png");
    pm.setMask(pm.createMaskFromColor(Qt::magenta));
    column = (Qtitan::GridTableColumn *)view->getColumnByModelColumnName("Reminder");
    column->setIcon(pm);
    column->setIconApperance(Qtitan::IconOnly | Qtitan::IconAlignLeft);
    column->setWidth(pm.width());
    column->setTextAlignment(Qt::AlignCenter);
    column->setHorSizingEnabled(false);
    column->setEditorType(Qtitan::PictureEditorType);
    column->editorRepository()->setEditable(false);
    column->editorRepository()->setAlignment(Qt::AlignCenter);
    ((GridPictureEditorRepository *)column->editorRepository())->setScaled(false);


    pm = QPixmap(":res/read.png");
    pm.setMask(pm.createMaskFromColor(Qt::magenta));
    column = (Qtitan::GridTableColumn *)view->getColumnByModelColumnName("Read");
    column->setIcon(pm);
    column->setIconApperance(Qtitan::IconOnly | Qtitan::IconAlignLeft);
    column->setWidth(pm.width());
    column->setTextAlignment(Qt::AlignCenter);
    column->setHorSizingEnabled(false);
    column->setEditorType(Qtitan::PictureEditorType);
    column->editorRepository()->setEditable(false);
    column->editorRepository()->setAlignment(Qt::AlignCenter);
    ((GridPictureEditorRepository *)column->editorRepository())->setScaled(false);

    pm = QPixmap(":res/status.png");
    pm.setMask(pm.createMaskFromColor(Qt::magenta));
    column = (Qtitan::GridTableColumn *)view->getColumnByModelColumnName("Status");
    column->setIcon(pm);
    column->setIconApperance(Qtitan::IconOnly | Qtitan::IconAlignLeft);
    column->setWidth(pm.width());    
    column->setHorSizingEnabled(false);
    column->setEditorType(Qtitan::PictureEditorType);
    column->editorRepository()->setEditable(false);
    column->editorRepository()->setAlignment(Qt::AlignCenter);
    ((GridPictureEditorRepository *)column->editorRepository())->setScaled(false);

    //Create settings widget
    QWidget* settings = new QWidget(this);
    QVBoxLayout* l = new QVBoxLayout(settings);
    m_autoWidthCheck = new QCheckBox(settings);
    m_autoWidthCheck->setText("Column auto width");
    connect(m_autoWidthCheck, SIGNAL(stateChanged(int)), this, SLOT(autoWidthStateChanged(int)));
    l->addWidget(m_autoWidthCheck);
    m_autoWidthCheck->setChecked(true);

    m_fastScrollCheck = new QCheckBox(settings);
    m_fastScrollCheck->setText("Fast scroll effect (new)");
    connect(m_fastScrollCheck, SIGNAL(stateChanged(int)), this, SLOT(fastScrollChanged(int)));
    l->addWidget(m_fastScrollCheck);
    m_fastScrollCheck->setChecked(true);

    setDemoWidget(m_grid, settings);
}

void Window::autoWidthStateChanged(int state)
{
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();
    view->options().setColumnAutoWidth(state == Qt::Checked);
}

void Window::fastScrollChanged(int state)
{
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();
    view->options().setFastScrollEffect(state == Qt::Checked);
}


