
#include <QtGui>
#include <QDirModel>
#include <QtSql>
#include <QtitanGrid.h>

#include "window.h"

Window::Window()
: DemoMainWindow("QtitanDataGrid")
{
    setWindowTitle(tr("Qtitan grid demo (Multi-selection)"));
    setGeometry(200, 250, 1000, 500);

    m_grid = new Qtitan::Grid();

    QString prefix = QApplication::applicationDirPath();
    prefix += "/..";

    QSqlError err;
    QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE", QString("database_demo"));

    db.setDatabaseName(prefix + "/demos/data/database.sqlite");
    db.setHostName("");
    db.setPort(-1);
    if (!db.open("", ""))
    {
        err = db.lastError();
        QSqlDatabase::removeDatabase(QString("database_demo"));
        QMessageBox::critical(0, "Demo Error", QString("Error: Can't open database - %1, error - %2").arg(prefix + "/demos/data/database.sqlite").arg(err.text()));
        QApplication::exit(1);
        return;
    }

    QSqlTableModel* model = new QSqlTableModel(m_grid, db);
    model->setTable("data");
    model->select();
    if (model->lastError().type() != QSqlError::NoError)
    {
        QMessageBox::critical(0, "Demo Error", "Error: SQL data base is not valid.");
        QApplication::exit(1);
        return;
    }
    model->setEditStrategy(QSqlTableModel::OnFieldChange);

    // Configure grid view
    m_grid->setViewType(Qtitan::Grid::BandedTableView);
    Qtitan::GridBandedTableView* view = m_grid->view<Qtitan::GridBandedTableView>();
    Qtitan::GridTableBand* band = view->addBand("Characteristics");
    band = view->addBand("Engineering");
    view->setModel(model);

    Qtitan::GridBandedTableColumn* column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Photo");
    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Registration");
    //column->setRowIndex(1);

    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Aircraft");
    ///column->setRowIndex(2);

    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Location");
    //column->setRowIndex(3);

    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Date");
    //column->setRowIndex(4);


    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Photo");
    column->setEditorType(Qtitan::ComboBoxPictureEditorType);
    column->setBandIndex(1);

    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("History");
    column->setEditorType(Qtitan::MemoEditorType);
    column->setBandIndex(1);

    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Info");
    column->setEditorType(Qtitan::MemoEditorType);
    column->setBandIndex(1);

    //Create settings widget
    QWidget* settings = new QWidget(this);
    QGridLayout *layout = new QGridLayout(settings);

    m_label = new QLabel(this);
    m_label->setText("Multi-select mode:");
    m_policyBox = new QComboBox(this);
    m_policyBox->addItem("Single Row");
    m_policyBox->addItem("Single Cell");
    m_policyBox->addItem("Multi Row");
    m_policyBox->addItem("Multi Row (With RubberBand selection)");
    m_policyBox->addItem("Multi Cell");
    m_policyBox->addItem("Multi Cell (With RubberBand selection)");
    connect(m_policyBox, SIGNAL(activated(int)), this, SLOT(newPolicyActivated(int)));
    m_showSelectedButton = new QPushButton();
    m_showSelectedButton->setText("Show Selected");
    connect(m_showSelectedButton, SIGNAL(clicked()), this, SLOT(showSelected()));

    m_fastScrollCheck = new QCheckBox(settings);
    m_fastScrollCheck->setText("Fast scroll effect (new)");
    connect(m_fastScrollCheck, SIGNAL(stateChanged(int)), this, SLOT(fastScrollChanged(int)));
    m_fastScrollCheck->setChecked(true);

    layout->addWidget(m_label, 0, 0);
    layout->addWidget(m_policyBox, 0, 1);
    layout->addWidget(m_showSelectedButton, 1, 1);
    layout->addWidget(m_fastScrollCheck, 2, 0, 1, 2);

    setDemoWidget(m_grid, settings);
}

void Window::newPolicyActivated(int index)
{
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();
    switch (index)
    {
    case 0:
        view->options().setSelectionPolicy(Qtitan::SelectSingleRow);
        break;
    case 1:
        view->options().setSelectionPolicy(Qtitan::SelectSingleCell);
        break;
    case 2:
        view->options().setSelectionPolicy(Qtitan::SelectMultiRow);
        view->options().setRubberBandSelection(false);
        break;
    case 3:
        view->options().setSelectionPolicy(Qtitan::SelectMultiRow);
        view->options().setRubberBandSelection(true);
        break;
    case 4:
        view->options().setSelectionPolicy(Qtitan::SelectMultiCell);
        view->options().setRubberBandSelection(false);
        break;
    case 5:
        view->options().setSelectionPolicy(Qtitan::SelectMultiCell);
        view->options().setRubberBandSelection(true);
        break;
    default:
        view->options().setSelectionPolicy(Qtitan::SelectSingleRow);
    }
}

void Window::showSelected()
{
}

void Window::fastScrollChanged(int state)
{
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();
    view->options().setFastScrollEffect(state == Qt::Checked);
}
