
#include <QtGui>
#include <QDirModel>

#include "window.h"
#include "stardelegate.h"
#include "starrating.h"

Window::Window()
: DemoMainWindow("QtitanDataGrid")
{
    setWindowTitle(tr("Qtitan grid demo (Qt delegates)"));
    setGeometry(50, 50, 800, 500);
    m_tabs = new QTabWidget(this);
    createStandartGrid();
    createSmartGrid();

    //Create settings widget
    QWidget* settings = new QWidget(this);
    QVBoxLayout* l = new QVBoxLayout(settings);

    m_autoWidthCheck = new QCheckBox(settings);
    m_autoWidthCheck->setText("Column auto width");
    connect(m_autoWidthCheck, SIGNAL(stateChanged(int)), this, SLOT(autoWidthStateChanged(int)));
    l->addWidget(m_autoWidthCheck);
    m_autoWidthCheck->setChecked(true);
    m_fastScrollCheck = new QCheckBox(settings);
    m_fastScrollCheck->setText("Fast scroll effect (new)");
    connect(m_fastScrollCheck, SIGNAL(stateChanged(int)), this, SLOT(fastScrollChanged(int)));
    l->addWidget(m_fastScrollCheck);  
    m_fastScrollCheck->setChecked(true);

    setDemoWidget(m_tabs, settings);
}

void Window::editorEditingStarted(QObject *, EditorEventArgs* args)
{
    if (args->dataBinding()->columnName() != "Cast")
        return;
    //Customize grid editor here
    Qtitan::TableComboBox* box = static_cast<Qtitan::TableComboBox *>(args->editor()->editWidget());
    Qtitan::Grid* grid = box->grid();
    Qtitan::GridTableView* view = grid->view<Qtitan::GridTableView>();
    view->options().setRowAutoHeight(true);
    view->options().setNewRowText(tr("Click here to add new actor"));
    Qtitan::GridTableColumn* column = (Qtitan::GridTableColumn *)view->getColumnByModelColumnName("Actor photo");
    if (column)
        column->setEditorType(Qtitan::PictureEditorType);
}

void Window::autoWidthStateChanged(int state)
{
    Qtitan::GridTableView* standartView = m_standartGrid->view<Qtitan::GridTableView>();
    if (standartView)
        standartView->options().setColumnAutoWidth(state == Qt::Checked);
    
    Qtitan::GridTableView* smartView = m_smartGrid->view<Qtitan::GridTableView>();
    if (smartView)
        smartView->options().setColumnAutoWidth(state == Qt::Checked);
}

void Window::fastScrollChanged(int state)
{
    Qtitan::GridTableView* standartView = m_standartGrid->view<Qtitan::GridTableView>();
    if (standartView)
        standartView->options().setFastScrollEffect(state == Qt::Checked);
    
    Qtitan::GridTableView* smartView = m_smartGrid->view<Qtitan::GridTableView>();
    if (smartView)
        smartView->options().setFastScrollEffect(state == Qt::Checked);
}

/* StandartDelegateDemoModel */
StandartDelegateDemoModel::StandartDelegateDemoModel(QObject *parent) 
: QAbstractItemModel(parent)
{
    m_values.push_back(DataItem("Mass in B-Minor", "Baroque", "J.S. Bach", 5));
    m_values.push_back(DataItem("Three More Foxes", "Jazz", "Maynard Ferguson", 4));
    m_values.push_back(DataItem("Sex Bomb", "Pop", "Tom Jones", 3));
    m_values.push_back(DataItem("Barbie Girl", "Pop", "Aqua", 5));
} 

StandartDelegateDemoModel::~StandartDelegateDemoModel()
{
}

QVariant StandartDelegateDemoModel::headerData(int section, Qt::Orientation orientation,
                                      int role) const
{
    Q_UNUSED(orientation);
    Q_UNUSED(role);
    switch (section)
    {
    case 0:
        {
            return QString("Title");
        }
        break;
    case 1:
        {
            return QString("Genre");
        }
        break;
    case 2:
        {
            return QString("Artist");
        }
        break;
    case 3:
        {
            return QString("Rating");
        }
        break;
    default:
        return QVariant();
    }
}

QModelIndex StandartDelegateDemoModel::parent(const QModelIndex & /*child*/) const 
{
    return QModelIndex();
} 

bool StandartDelegateDemoModel::hasChildren(const QModelIndex &parent) const 
{  
    if (parent.model() == this || !parent.isValid()) 
    {
        return rowCount(parent) > 0 && columnCount(parent) > 0;
    }
    return false;
} 

int StandartDelegateDemoModel::rowCount(const QModelIndex &parent) const 
{
    if (parent.isValid())
        return 0;
    return m_values.size();
} 

int StandartDelegateDemoModel::columnCount(const QModelIndex &parent) const 
{
    if (parent.isValid())
        return 0;
    return 4;
}

QModelIndex StandartDelegateDemoModel::index(int row, int column, const QModelIndex &parent) const 
{
    if (parent.isValid())                                   
        return QModelIndex();

    if (row < 0 || row >= rowCount(parent))
        return QModelIndex();

    if (column < 0 || column >= columnCount(parent)) 
        return QModelIndex();

    return createIndex(row, column, (void*)NULL);
}

QVariant StandartDelegateDemoModel::data(const QModelIndex &index, int role) const 
{  
    if (!index.isValid())
        return QVariant();
    
    if (index.row() < 0 || index.row() >= rowCount(index.parent()))
        return QVariant();
    
    if (index.column() < 0 || index.column() >= columnCount(index.parent()))
        return QVariant();
    
    if (role == Qt::DisplayRole || role == Qt::EditRole)        
    {    
        switch (index.column())
        {
        case 0:
            {
                return m_values[index.row()].title;
            }
            break;
        case 1:
            {
                return m_values[index.row()].genre;
            }
            break;
        case 2:
            {
                return m_values[index.row()].artist;
            }
            break;
        case 3:
            {
                return qVariantFromValue(StarRating(m_values[index.row()].rating));
            }
            break;
        default:
            break;
        }
    }
    return QVariant();
}

bool StandartDelegateDemoModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    if (!index.isValid())
        return false;
    
    if (index.row() < 0 || index.row() >= rowCount(index.parent()))
        return false;
    
    if (index.column() < 0 || index.column() >= columnCount(index.parent()))
        return false;

    if (role != Qt::EditRole)
        return false;

    switch (index.column())
    {
    case 0:
        {
            m_values[index.row()].title = value.toString();
        }
        break;
    case 1:
        {
            m_values[index.row()].genre = value.toString();
        }
        break;
    case 2:
        {
            m_values[index.row()].artist = value.toString();
        }
        break;
    case 3:
        {
            StarRating starRating = qVariantValue<StarRating>(value);
            m_values[index.row()].rating = starRating.starCount();
        }
        break;
    default:
        break;
    }

    emit dataChanged(index, index);
    return true;
}

Qt::ItemFlags StandartDelegateDemoModel::flags(const QModelIndex &index) const 
{  
    if (!index.isValid())
        return false;
    return Qt::ItemIsEnabled | Qt::ItemIsEditable;
}                

SmartDelegateDemoModel::SmartDelegateDemoModel(QObject *parent, QSqlDatabase db)
: QSqlTableModel(parent, db)
{
}

QVariant SmartDelegateDemoModel::data(const QModelIndex &index, int role) const
{
    if (index.column() == 8)
    {
        QVariant v = QSqlTableModel::data(index, role);
        return qVariantFromValue(StarRating(v.toInt()));
    }
    return QSqlTableModel::data(index, role);
}

bool SmartDelegateDemoModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    if (index.column() == 8)
    {
        StarRating starRating = qVariantValue<StarRating>(value);
        return QSqlTableModel::setData(index, starRating.starCount(), role);
    }
    return QSqlTableModel::setData(index, value, role);
}

/* CountriesDemoModel */
CountriesDemoModel::CountriesDemoModel(QObject *parent, QSqlDatabase db)
: QSqlTableModel(parent, db)
{
}

QVariant CountriesDemoModel::data(const QModelIndex &index, int role) const
{
    if (role == Qt::DecorationRole)
    {
        QModelIndex pixmapindex = QSqlTableModel::index(index.row(), 1, index.parent());
        QPixmap pixmap;
        qvariant_to_pixmap(QSqlTableModel::data(pixmapindex, Qt::DisplayRole), pixmap);
        return pixmap;
    }
    return QSqlTableModel::data(index, role);
}


/* Window */
void Window::createStandartGrid()
{
    m_standartGrid = new Qtitan::Grid();
    m_standartGrid->setViewType(Qtitan::Grid::TableView);
    Qtitan::GridTableView* view = m_standartGrid->view<Qtitan::GridTableView>();
    view->options().setBorderWidth(0);

    StandartDelegateDemoModel* model = new StandartDelegateDemoModel(m_standartGrid);
    view->setModel(model);
    Qtitan::GridTableColumn* column = (Qtitan::GridTableColumn *)view->getColumn(3);
    column->setEditorType(Qtitan::DelegateAdapterEditorType);
    GridDelegateAdapterEditorRepository* adapter = static_cast<GridDelegateAdapterEditorRepository *>(column->editorRepository());
    adapter->setDelegate(new StarDelegate);

    m_tabs->insertTab(0, m_standartGrid, tr("Standart"));
}

void Window::createSmartGrid()
{
    m_smartGrid = new Qtitan::Grid();
    m_tabs->insertTab(1, m_smartGrid, tr("Smart"));

    QString prefix = QApplication::applicationDirPath();
    prefix += "/..";

    QSqlError err;
    QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE", QString("database_demo"));

    db.setDatabaseName(prefix + "/demos/data/films.sqlite");
    db.setHostName("");
    db.setPort(-1);
    if (!db.open("", ""))
    {
        err = db.lastError();
        QSqlDatabase::removeDatabase(QString("database_demo"));
        QMessageBox::critical(0, "Demo Error", QString("Error: Can't open database - %1, error - %2").arg(prefix + "/demos/data/films.sqlite").arg(err.text()));
        QApplication::exit(1);
        return;
    }

    QSqlTableModel* model = new SmartDelegateDemoModel(m_smartGrid, db);
    model->setTable("Films");
    model->select();
    if (model->lastError().type() != QSqlError::NoError)
    {
        QMessageBox::critical(0, "Demo Error", "Error: SQL data base is not valid.");
        QApplication::exit(1);
        return;
    }

    QSqlTableModel* countriesModel = new CountriesDemoModel(m_smartGrid, db);
    countriesModel->setTable("Countries");
    countriesModel->select();
    if (countriesModel->lastError().type() != QSqlError::NoError)
    {
        QMessageBox::critical(0, "Demo Error", "Error: SQL data base is not valid.");
        QApplication::exit(1);
        return;
    }

    // Configure grid view
    m_smartGrid->setViewType(Qtitan::Grid::TableView);
    Qtitan::GridTableView* view = m_smartGrid->view<Qtitan::GridTableView>();
    view->options().setBorderWidth(0);
    view->options().setRowAutoHeight(true);
    view->options().setScrollRowStyle(Qtitan::ScrollStyleWeb);

    connect(view, SIGNAL(editorEditingStarted(QObject *, EditorEventArgs *)),
        this, SLOT(editorEditingStarted(QObject *, EditorEventArgs *)));

    view->setModel(model);

    Qtitan::GridTableColumn* column = (Qtitan::GridTableColumn *)view->getColumnByModelColumnName("Goofs");
    column->setEditorType(Qtitan::MemoEditorType);

    column = (Qtitan::GridTableColumn *)view->getColumnByModelColumnName("Plot");
    column->setEditorType(Qtitan::MemoEditorType);

    column = (Qtitan::GridTableColumn *)view->getColumnByModelColumnName("Photo");
    column->setEditorType(Qtitan::PictureEditorType);

    column = (Qtitan::GridTableColumn *)view->getColumnByModelColumnName("Country");
    column->setEditorType(Qtitan::ComboBoxEditorType);
    column->editorRepository()->setEditorFocusPolicy(Qtitan::Always);
    column->dataBinding()->setRelationMode(ModelDataBinding::Relation);
    column->dataBinding()->setRelationModel(countriesModel);
    column->dataBinding()->setForeignKey("id");
    column->dataBinding()->setRoleMapping(Qt::DisplayRole, "Name");
    column->dataBinding()->setRoleMapping(Qt::DecorationRole, "Flag");


    column = (Qtitan::GridTableColumn *)view->getColumnByModelColumnName("Rating");
    column->setEditorType(Qtitan::DelegateAdapterEditorType);
    GridDelegateAdapterEditorRepository* adapter = static_cast<GridDelegateAdapterEditorRepository *>(column->editorRepository());
    adapter->setDelegate(new StarDelegate);

    QSqlTableModel* actorsModel = new QSqlTableModel(0, db);
    actorsModel->setTable("Actors");
    actorsModel->select();
    column = (Qtitan::GridTableColumn *)view->getColumnByModelColumnName("Cast");
    column->setEditorType(Qtitan::TableEditorType);
    column->dataBinding()->setRelationMode(ModelDataBinding::MasterDetailsRelation);
    column->dataBinding()->setRelationModel(actorsModel);
    column->dataBinding()->setForeignKey("film_id");
}
