
#include <QtGui>
#include <QDirModel>
#include <QtitanGrid.h>

#include "window.h"

/* Window */
Window::Window()
: DemoMainWindow("QtitanDataGrid")
{
    setWindowTitle(tr("Qtitan grid demo (Summaries)"));
    setGeometry(150, 150, 700, 500);

    m_grid = new Qtitan::Grid();
    SummaryDemoModel* model = new SummaryDemoModel(m_grid);

    // Configure grid view
    m_grid->setViewType(Qtitan::Grid::BandedTableView);
    Qtitan::GridBandedTableView* view = m_grid->view<Qtitan::GridBandedTableView>();
    view->addBand("Section 1");
    view->addBand("Section 2");
    view->addBand("Section 3");
    view->options().setFooterSummaryVisible(true);
    view->options().setGroupSummaryVisible(true);
    view->options().setGridLines(LinesHorizontal);
    view->setModel(model);

    Qtitan::GridBandedTableColumn* column = (Qtitan::GridBandedTableColumn *)view->getColumn(0);
    column->setBandIndex(0);

    //Configure the summary
    column->setDefaultGroupSummary(Qtitan::SummaryMax);
    column = (Qtitan::GridBandedTableColumn *)view->getColumn(1);
    column->setBandIndex(1);

    //Configure the summary
    column->setDefaultGroupSummary(Qtitan::SummaryAvg);
    column->setFooterSummary(Qtitan::SummaryAvg);
    column = (Qtitan::GridBandedTableColumn *)view->getColumn(2);
    column->setBandIndex(1);

    //Configure the summary
    column->setDefaultGroupSummary(Qtitan::SummaryCount);
    column->setFooterSummary(Qtitan::SummaryCount);
    column = (Qtitan::GridBandedTableColumn *)view->getColumn(3);
    column->setBandIndex(1);

    column = (Qtitan::GridBandedTableColumn *)view->getColumn(4);
    column->setBandIndex(2);

    column = (Qtitan::GridBandedTableColumn *)view->getColumn(4);
    column->setGroupIndex(0);

    //Configure the summary
    column->setDefaultGroupSummary(Qtitan::SummaryMin);
    column->setFooterSummary(Qtitan::SummaryMax);
    view->expandAll();

    QWidget* settings = new QWidget(this);
    QVBoxLayout* l = new QVBoxLayout(settings);
    m_autoWidthCheck = new QCheckBox(settings);
    l->addWidget(m_autoWidthCheck);
    m_autoWidthCheck->setText("Column auto width");
    connect(m_autoWidthCheck, SIGNAL(stateChanged(int)), this, SLOT(autoWidthStateChanged(int)));

    m_fastScrollCheck = new QCheckBox(settings);
    l->addWidget(m_fastScrollCheck);
    m_fastScrollCheck->setText("Fast scroll effect (new)");
    connect(m_fastScrollCheck, SIGNAL(stateChanged(int)), this, SLOT(fastScrollChanged(int)));
    m_fastScrollCheck->setChecked(false);
    setDemoWidget(m_grid, settings);
}

void Window::autoWidthStateChanged(int state)
{
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();
    view->options().setColumnAutoWidth(state == Qt::Checked);
}

void Window::fastScrollChanged(int state)
{
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();
    view->options().setFastScrollEffect(state == Qt::Checked);
}

/* SummaryDemoModel */
SummaryDemoModel::SummaryDemoModel(QObject *parent) 
: QAbstractItemModel(parent)
{
    m_values.resize(100);
    int y = 2000;
    int m = 1;
    int d = 1;
    for (int i = 0; i < m_values.size(); ++i)
    {
        m_values[i].v0 = true;
        m_values[i].v1 = i;
        m_values[i].v2 = QString("String-1 = %1").arg(i);
        m_values[i].v3 = QString("String-2 = %1").arg(i);
        if (d > 28)
        {
            d = 1;
            m++;
            if (m > 12)
            {
                m = 1;
                y++;
            }
        }
        m_values[i].v4 = QDate(y, m, d);
        if (!(i % 10))
            d++;
    }
} 

SummaryDemoModel::~SummaryDemoModel()
{
} 

QVariant SummaryDemoModel::headerData(int section, Qt::Orientation orientation,
                                      int role) const
{
    Q_UNUSED(orientation);
    Q_UNUSED(role);
    switch (section)
    {
    case 0:
        {
            return QString("Boolean");
        }
        break;
    case 1:
        {
            return QString("Integer");
        }
        break;
    case 2:
        {
            return QString("String");
        }
        break;
    case 3:
        {
            return QString("String");
        }
        break;
    case 4:
        {
            return QString("Date");
        }
        break;
    }
    return QVariant();
}

QModelIndex SummaryDemoModel::parent(const QModelIndex & /*child*/) const 
{
    return QModelIndex();
} 

bool SummaryDemoModel::hasChildren(const QModelIndex &parent) const 
{  
    if (parent.model() == this || !parent.isValid()) 
    {
        return rowCount(parent) > 0 && columnCount(parent) > 0;
    }
    return false;
} 

int SummaryDemoModel::rowCount(const QModelIndex &parent) const 
{
    if (parent.isValid())
        return 0;
    return m_values.size();
} 

int SummaryDemoModel::columnCount(const QModelIndex &parent) const 
{
    if (parent.isValid())
        return 0;
    return 5;
}

QModelIndex SummaryDemoModel::index(int row, int column, const QModelIndex &parent) const 
{
    if (parent.isValid())                                   
        return QModelIndex();

    if (row < 0 || row >= rowCount(parent))
        return QModelIndex();

    if (column < 0 || column >= columnCount(parent)) 
        return QModelIndex();

    return createIndex(row, column, (void*)NULL);
}

QVariant SummaryDemoModel::data(const QModelIndex &index, int role) const 
{  
    if (!index.isValid())
        return QVariant();
    
    if (index.row() < 0 || index.row() >= rowCount(index.parent()))
        return QVariant();
    
    if (index.column() < 0 || index.column() >= columnCount(index.parent()))
        return QVariant();
    
    if (role == Qt::DisplayRole || role == Qt::EditRole)        
    {    
        switch (index.column())
        {
        case 0:
            {
                return m_values[index.row()].v0;
            }
            break;
        case 1:
            {
                return m_values[index.row()].v1;
            }
            break;
        case 2:
            {
                return m_values[index.row()].v2;
            }
            break;
        case 3:
            {
                return m_values[index.row()].v3;
            }
            break;
        case 4:
            {
                return m_values[index.row()].v4;
            }
            break;
        }
    }
    else if (role == Qt::CheckStateRole)
    {
        switch (index.column())
        {
        case 0:
            {
                return m_values[index.row()].v0;
            }
            break;
        }
    }
    return QVariant();
}

bool SummaryDemoModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    if (!index.isValid())
        return false;
    
    if (index.row() < 0 || index.row() >= rowCount(index.parent()))
        return false;
    
    if (index.column() < 0 || index.column() >= columnCount(index.parent()))
        return false;

    if (role != Qt::EditRole)
        return false;

    switch (index.column())
    {
    case 0:
        {
            m_values[index.row()].v0 = value.toBool();
        }
        break;
    case 1:
        {
            m_values[index.row()].v1 = value.toInt();
        }
        break;
    case 2:
        {
            m_values[index.row()].v2 = value.toString();
        }
        break;
    case 3:
        {
            m_values[index.row()].v3 = value.toString();
        }
        break;
    case 4:
        {
            m_values[index.row()].v4 = value.toDate();
        }
        break;
    }

    emit dataChanged(index, index);
    return true;
}

Qt::ItemFlags SummaryDemoModel::flags(const QModelIndex &index) const 
{  
    if (!index.isValid())
        return false;

    //if (index.column() == 0)
    //{
    //    return Qt::ItemIsEnabled | Qt::ItemIsEditable | Qt::ItemIsUserCheckable;
    //}
    //else if (index.column() == 4)
    //{
    //    return Qt::ItemIsEnabled | Qt::ItemIsEditable;
    //}
    return Qt::ItemIsEnabled | Qt::ItemIsEditable;
}                

