
#include <QtGui>
#include <QDirModel>
#include <QtSql>
#include <QtitanGrid.h>

#include "window.h"


Window::Window()
: DemoMainWindow("QtitanDataGrid")
{
    setWindowTitle(tr("Qtitan grid demo (Web like scrolling mode)"));
    setGeometry(50, 50, 800, 500);

    m_grid = new Qtitan::Grid();

    QString prefix = QApplication::applicationDirPath();
    prefix += "/..";

    QSqlError err;
    QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE", QString("database_demo"));

    db.setDatabaseName(prefix + "/demos/data/database.sqlite");
    db.setHostName("");
    db.setPort(-1);
    if (!db.open("", ""))
    {
        err = db.lastError();
        QSqlDatabase::removeDatabase(QString("database_demo"));
        QMessageBox::critical(0, "Demo Error", QString("Error: Can't open database - %1, error - %2").arg(prefix + "/demos/data/database.sqlite").arg(err.text()));
        QApplication::exit(1);
        return;
    }

    QSqlTableModel* model = new QSqlTableModel(m_grid, db);
    model->setTable("data");
    model->select();
    if (model->lastError().type() != QSqlError::NoError)
    {
        QMessageBox::critical(0, "Demo Error", "Error: SQL data base is not valid.");
        QApplication::exit(1);
        return;
    }

    // Configure grid view
    m_grid->setViewType(Qtitan::Grid::BandedTableView);
    Qtitan::GridBandedTableView* view = m_grid->view<Qtitan::GridBandedTableView>();
    view->options().setScrollRowStyle(Qtitan::ScrollStyleWeb);
    view->options().setGroupsHeader(false);

    view->options().setGridLineWidth(1);
    view->options().setGridLineColor(Qt::lightGray);
    view->options().setFastScrollEffect(false);

    Qtitan::GridTableBand* band = view->addBand("Characteristics");
    band = view->addBand("Engineering");
    view->setModel(model);

    Qtitan::GridBandedTableColumn* column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Photo");
    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Registration");
    column->setRowIndex(1);
    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Aircraft");
    column->setRowIndex(2);
    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Location");
    column->setRowIndex(3);
    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Date");
    column->setRowIndex(4);

    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Photo");
    column->setEditorType(Qtitan::PictureEditorType);
    Qtitan::GridPictureEditorRepository* pictureEditor = (Qtitan::GridPictureEditorRepository *)column->editorRepository();
    Q_UNUSED(pictureEditor);
    column->setBandIndex(1);
    column->setRowSpan(5);

    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("History");
    column->setEditorType(Qtitan::MemoEditorType);
    Qtitan::GridMemoEditorRepository* memoEditor = (Qtitan::GridMemoEditorRepository *)column->editorRepository();
    column->setBandIndex(1);
    column->setRowSpan(5);
    column = (Qtitan::GridBandedTableColumn *)view->getColumnByModelColumnName("Info");
    column->setEditorType(Qtitan::MemoEditorType);
    memoEditor = (Qtitan::GridMemoEditorRepository *)column->editorRepository();
    column->setBandIndex(1);
    column->setRowSpan(5);

    //Create settings widget
    QWidget* settings = new QWidget(this);
    QVBoxLayout* l = new QVBoxLayout(settings);

    m_autoWidthCheck = new QCheckBox(settings);
    m_autoWidthCheck->setText("Column auto width");
    connect(m_autoWidthCheck, SIGNAL(stateChanged(int)), this, SLOT(autoWidthStateChanged(int)));
    l->addWidget(m_autoWidthCheck);
    m_autoWidthCheck->setChecked(true);

    m_fastScrollCheck = new QCheckBox(settings);
    m_fastScrollCheck->setText("Fast scroll effect (new)");
    connect(m_fastScrollCheck, SIGNAL(stateChanged(int)), this, SLOT(fastScrollChanged(int)));
    l->addWidget(m_fastScrollCheck);
    
    setDemoWidget(m_grid, settings);
}

void Window::autoWidthStateChanged(int state)
{
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();
    view->options().setColumnAutoWidth(state == Qt::Checked);
}

void Window::fastScrollChanged(int state)
{
    Qtitan::GridTableView* view = m_grid->view<Qtitan::GridTableView>();
    view->options().setFastScrollEffect(state == Qt::Checked);
}

