/****************************************************************************
**
** Qtitan Library by Developer Machines (Advanced Grid for Qt)
** 
** Copyright (c) 2009-2011 Developer Machines (http://www.devmachines.com)
**           ALL RIGHTS RESERVED
** 
**  The entire contents of this file is protected by copyright law and
**  international treaties. Unauthorized reproduction, reverse-engineering
**  and distribution of all or any portion of the code contained in this
**  file is strictly prohibited and may result in severe civil and 
**  criminal penalties and will be prosecuted to the maximum extent 
**  possible under the law.
**
**  RESTRICTIONS
**
**  THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED
**  FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE
**  COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE
**  AVAILABLE TO OTHER INDIVIDUALS WITHOUT WRITTEN CONSENT
**  AND PERMISSION FROM DEVELOPER MACHINES
**
**  CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON
**  ADDITIONAL RESTRICTIONS.
**
****************************************************************************/

#ifndef QTN_GRID_H
#define QTN_GRID_H

#include <QtGui/QIcon>
#include <QtGui/QTabWidget>
#include <QtGui/QStyleOption>
#ifndef QTN_NOUSE_XML_MODULE
#include <QXmlStreamReader>
#include <QXmlStreamWriter>
#endif
#include "QtnGridDef.h"
#include "QtnGridGraphics.h"
#include "QtnGridModelController.h"

namespace Qtitan
{

    class GridViewBase;
    class GridHeaderPanelGraphic;
    class GridFooterPanelGraphic;
    class GridRowsGraphic;
    class GridFieldChooserWindow;
    class GridFloatToolWidget;
    class DragColumnWidget;
    class ColumnArrowWidget;
    class GridColumnElementGraphic;
    class GridFieldChooserProvider;
    class GridViewOptions;
    class EditorEventArgs;

    enum GridScrollStyle
    {
	    ScrollStyleNormal,
	    ScrollStyleWeb
    };

    enum LayoutZone
    {
        ZoneUndefined,
        ZoneLeft,
        ZoneRight,
        ZoneUp,
        ZoneDown
    };

    enum IconApperanceFlag
    {
        IconInvisible = 0x0,
        IconOnly = 0x1,
        IconAlignLeft = 0x2,
        IconAlignRight = 0x4
    };

    Q_DECLARE_FLAGS(IconApperance, IconApperanceFlag)

    class QTITAN_EXPORT Grid: public GridGraphicSite
    {
        Q_OBJECT
        Q_CLASSINFO("Version", "1.2")
        Q_ENUMS(GridViewType)
        Q_PROPERTY(GridViewType viewType READ viewType WRITE setViewType)
        Q_PROPERTY(GridViewBase* view READ viewref)
    public:
        enum GridViewType
        {
            UndefinedView = 0,
            TableView = 1,
            BandedTableView = 2,
        };
        Grid(QWidget* parent = 0);
        virtual ~Grid();
        GridViewType viewType() const;
        void setViewType(GridViewType type);
        template<typename ViewType>
        ViewType* view() const { Q_ASSERT(viewref() != Q_NULL); return qobject_cast<ViewType *>(viewref()); }
#ifndef QTN_NOUSE_XML_MODULE
        bool saveLayoutToFile(const QString& fileName);
        bool loadLayoutFromFile(const QString& fileName);
        bool saveLayoutToDevice(QIODevice* device);
        bool loadLayoutFromDevice(QIODevice* device);
#endif
    protected:
        GridViewBase* viewref() const;
        void setRootView(GridViewBase* view);
        GridViewBase* m_view;
        int m_viewType;
    private:
    };


    class GridViewGraphic: public GridScrollableGraphicPrimitive
    {
    public:
        GridViewGraphic(GridViewBase* view);
        virtual void requestUpdateLayout(bool recalculateHeight = false) = 0;
    protected:
        virtual void paintEvent(QPainter& painter);
        GridViewBase* m_view;
    };

    enum GridChangeFlag
    {
        NoChangesFlag = 0x0000,
        ColumnChangedFlag = 0x0001,
        ControllerChangedFlag = 0x0002,
        RequestUpdateLayout = 0x0004,
        RESERVEDFLAG0 = 0x0008,
        RESERVEDFLAG1 = 0x0010,
        RESERVEDFLAG2 = 0x0020,
        RESERVEDFLAG3 = 0x0040,
        RESERVEDFLAG4 = 0x0080,
    };

    Q_DECLARE_FLAGS(GridChangeFlags, GridChangeFlag)


    class GridColumnBase;

    class QTITAN_EXPORT GridViewBase: public QObject
    {
        Q_OBJECT
    public:
        friend class Grid;
        friend class GridViewGraphic;
        friend class GridHeaderPanelGraphic;
        friend class GridGroupPanelGraphic;
        friend class GridFooterPanelGraphic;
        friend class GridRowsGraphic;
        friend class GridColumnBase;
        friend class GridTableColumn;
        friend class GridColumnGraphic;
        friend class GridColumnChooseWidget;
        friend class GridTableColumnQuickButtonGraphic;
        friend class QuickCustomizationPopup;
        friend class GridTableDragColumnObject;
        friend class GridViewOptions;
        friend class GridTableViewOptions;
        friend class GridEditor;
        GridColumnBase* addColumn(const QString& modelColumnName, const QString& caption = QString());
        GridColumnBase* addColumn(int modelColumn, const QString& caption = QString());
        int getColumnCount() const;
        GridColumnBase* getColumn(int index) const;
        GridColumnBase* getColumnByModelColumn(int modelColumn) const;
        GridColumnBase* getColumnByModelColumnName(const QString& modelColumnName) const;
        GridColumnBase* getColumnByCaption(const QString& caption) const;
        bool removeColumn(int index);
        void removeColumns();
        virtual bool isColumnAvailable(const GridColumnBase* columnBase) const = 0;
	    void showFieldChooser();
        void hideFieldChooser();
        virtual void beginUpdate();
        virtual void endUpdate();
        void requestUpdateLayout();
	    Grid* grid() const;
        GridModelController* modelController() const;
        virtual bool showEditor() = 0;
        virtual bool closeEditor() = 0;
        virtual void hideEditor() = 0;
        virtual bool isEditing() const = 0;
        virtual void setModel(QAbstractItemModel* model, QModelIndex parent = QModelIndex(), bool createColumns = true);
        virtual QAbstractItemModel* model() const;
    public:
        typedef QList<GridColumnBase *> GridColumns;
        GridViewBase(Grid* grid);
        virtual ~GridViewBase();
    protected:
        virtual void init();
        virtual void columnsChanged() = 0;
        virtual void textAlignmentChanged(GridColumnBase* columnBase) = 0;
	    GridViewGraphic* graphic() const;
        GridHeaderPanelGraphic* headerPanelGraphic() const;
        GridFooterPanelGraphic* footerPanelGraphic() const;
        GridRowsGraphic* rowsGraphic() const;
        virtual int getColumnIndex(const GridColumnBase* columnBase) const;
        virtual int getColumnSortIndex(const GridColumnBase* columnBase) const;
        virtual void setColumnSortIndex(const GridColumnBase* columnBase, int sortIndex);
        virtual SortOrder getColumnSortOrder(const GridColumnBase* columnBase) const;
        virtual void setColumnSortOrder(GridColumnBase* columnBase, 
            SortOrder order, bool caseSensitivity, bool localeAware);
        virtual int columnGroupIndex(const GridColumnBase* columnBase) const = 0;
        virtual void setColumnGroupIndex(GridColumnBase* columnBase, int groupIndex, 
            bool caseSensitivity, bool localeAware) = 0;
        virtual bool getColumnVisible(const GridColumnBase* columnBase) const = 0;
        virtual void setColumnVisible(GridColumnBase* columnBase, bool visible) = 0;
        virtual GridColumnBase* createColumn(int modelColumn, const QString& caption) = 0;
        virtual GridViewOptions* createOptions() = 0;
	    virtual GridViewGraphic* createGraphic() = 0;
        virtual GridHeaderPanelGraphic* createHeaderPanelGraphic() = 0;
        virtual GridFooterPanelGraphic* createFooterPanelGraphic() = 0;
        virtual GridRowsGraphic* createRowsGraphic() = 0;
        virtual void columnCreated(const GridColumnBase* column) = 0;
        virtual const QFont& getFont(const GridColumnBase* column) const;
        virtual const QPen& getPen(const GridColumnBase* column) const;
        GridViewOptions* optionsref() const;
        void updateBorder();
        virtual void raiseQueryRelation(RelationEventArgs* args);
        virtual void raiseGroupExpanding(ModelGroup* group, bool& allow);
        virtual void raiseGroupExpanded(ModelGroup* group);
        virtual void raiseGroupCollapsing(ModelGroup* group, bool& allow);
        virtual void raiseGroupCollapsed(ModelGroup* group);
        void raiseEditorCreated(EditorEventArgs* args);
        void raiseEditorDestroying(EditorEventArgs* args);
        void raiseEditorEditingStarted(EditorEventArgs* args);
        void raiseEditorEditingStoping(EditorEventArgs* args);
#ifndef QTN_NOUSE_XML_MODULE
    protected:
        virtual bool saveLayoutToXML(QXmlStreamWriter* xmlwriter) = 0;
        virtual bool loadLayoutFromXML(QXmlStreamReader* xmlreader) = 0;
#endif
    protected:
        virtual void focusInEvent(QFocusEvent *);
        virtual void focusOutEvent(QFocusEvent *);
    protected:
        GridFieldChooserWindow* fieldChooserWindow() const;
        void changeGrouping(GridColumnBase* columnBase, int groupIndex, bool caseSensitivity, bool localeAware);
        virtual int getColumnHeight(GridColumnBase* columnBase) const = 0;
        bool isUpdating() const;
        GridColumns m_columns;
        GridColumns m_columnslist;
        GridChangeFlags m_changeFlags;
    protected Q_SLOTS:
        virtual void controllerChanged() = 0;
    private:
	    Grid* m_grid;
        GridModelController* m_modelController;
        GridFieldChooserWindow* m_fieldChooserWindow;
        GridViewOptions* m_options;
	    GridViewGraphic* m_viewGP;
        GridHeaderPanelGraphic* m_headerPanelGP;
        GridFooterPanelGraphic* m_footerPanelGP;
        GridRowsGraphic* m_rowsGP;
        int m_updateStateStack;
    Q_SIGNALS:
        void columnsUpdated(QObject* sender);
        void groupExpanding(QObject* sender, ModelGroup* group, bool& allow);
        void groupExpanded(QObject* sender, ModelGroup* group);
        void groupCollapsing(QObject* sender, ModelGroup* group, bool& allow);
        void groupCollapsed(QObject* sender, ModelGroup* group);
        void editorCreated(QObject* sender, EditorEventArgs* args);
        void editorDestroying(QObject* sender, EditorEventArgs* args);
        void editorEditingStarted(QObject* sender, EditorEventArgs* args);
        void editorEditingStoping(QObject* sender, EditorEventArgs* args);
        void relationConnecting(QObject* sender, RelationEventArgs* args);
    };

    class GridColumnGraphic: public GridGraphicPrimitive
    {
        friend class GridColumnElementGraphic;
        friend class DragColumnWidget;
    public:
	    GridColumnGraphic(GridColumnBase* column);
    protected:
        GridColumnBase* m_column;
    };

    class QTITAN_EXPORT GridColumnBase: public QObject
    {
    public:
        friend class GridColumnGraphic;
        friend class DragColumnWidget;
        friend class GridViewBase;
        friend class GridColumnElementGraphic;
        friend class GridTableViewOptions;
        friend class GridTableColumnGraphic;
        GridColumnBase(GridViewBase* view, const QString& caption);
        virtual ~GridColumnBase();
        int index() const;
        ModelDataBinding* dataBinding() const;
        const QString& caption() const;
        void setCaption(const QString& caption);
        virtual int minWidth() = 0; 
        virtual int maxWidth() = 0; 
        virtual bool isShowingFirst() const = 0;
        virtual bool isShowingLast() const = 0;
        Qt::Alignment textAlignment() const;
        void setTextAlignment(Qt::Alignment alignment);
        Qt::TextElideMode textElideMode() const;
        void setTextElideMode(Qt::TextElideMode mode);
        void setIcon(const QIcon& icon);
        const QIcon& icon() const;
        IconApperance iconApperance() const;
        void setIconApperance(IconApperance apperance);
        bool isVisible() const;
        void setVisible(bool visible);
    protected:
        virtual void init();
        template<typename ViewType>
        inline ViewType* view() const { Q_ASSERT(m_view != Q_NULL); return static_cast<ViewType *>(m_view); }
        virtual void initStyleOption(QStyleOptionHeader& styleOption) = 0;
	    GridColumnGraphic* graphic() const;
        virtual void click(QMouseEvent* e) = 0;
        int checkWidth(int width);
        bool m_visible;
        GridColumnGraphic* m_gp;
    private:
        GridViewBase* m_view;
        ModelDataBinding* m_dataBinding;
        QString m_caption;
        QIcon m_icon;
        IconApperance m_iconApperance;
        Qt::Alignment m_textAlignment;
        Qt::TextElideMode m_textElideMode;
    };

    class GridViewAutoUpdater
    {
    public:
        GridViewAutoUpdater(GridViewBase* view)
            : m_view(view)
        {
            m_view->beginUpdate();
        }

        ~GridViewAutoUpdater()
        {
            m_view->endUpdate();
        }
    private:
        GridViewBase* m_view;
    };

    class GridHeaderPanelGraphic: public GridScrollableGraphicPrimitive
    {
    public:
        GridHeaderPanelGraphic(GridGraphicPrimitive* parent);
        virtual bool groupPanelVisible() const = 0;
        virtual void setGroupPanelVisible(bool visible) = 0;
    };

    class GridGroupPanelGraphic: public GridGraphicPrimitive
    {
    public:
        friend class GridViewOptions;
        friend class GridTableDragColumnObject;
        GridGroupPanelGraphic(GridViewBase* view, GridGraphicPrimitive* parent);
        const QVector<GridColumnElementGraphic *>& columnElements() const;
        GridViewBase* view() const;
    protected:
        virtual void paintEvent(QPainter& painter);
        virtual bool startDragEvent(const QPoint& p);
        GridColumnElementGraphic* columnElement(int groupIndex);
    protected:
        QVector<GridColumnElementGraphic *> m_colElements;
    private:
        GridViewBase* m_view;
    };

    class GridFooterPanelGraphic: public GridGraphicPrimitive
    {
    public:
        GridFooterPanelGraphic(GridViewBase* view, const QRect& rect);
        virtual void updateBorder() = 0;
    protected:
        virtual void paintEvent(QPainter& painter);
        GridViewBase* m_view;
    private:
    };

    class GridRowsGraphic: public GridScrollableGraphicPrimitive
    {
    public:
        GridRowsGraphic(GridViewBase* view);
    protected:
        virtual void paintEvent(QPainter& painter);
    private:
    };

    class GridColumnElementGraphic: public GridGraphicPrimitive
    {
    public:
        GridColumnElementGraphic(GridGraphicPrimitive* parent, GridColumnBase* columnBase);
        GridColumnBase* column() const;
    protected:
        virtual void mouseOverEvent(const QPoint& pos);
        virtual void mouseOutEvent();
        virtual void mousePressEvent(const QPoint& point, QMouseEvent* e);
        virtual void mouseDoubleClickEvent(const QPoint& point, QMouseEvent* e);
        virtual void mouseReleaseEvent(const QPoint& point, QMouseEvent* e);
        virtual bool startDragDropEvent(const QPoint& p);
        virtual bool checkDragDropEvent(const QPoint& pos, GridDragDropObject* object);
        virtual GridDragDropObject* createDragDropObject(const QPoint& point);
        virtual void paintEvent(QPainter& painter);
    private:
        GridColumnBase* m_column;
    };

    class GridColumnChooseArea: public GridScrollableGraphicPrimitive
    {
    public:
        GridColumnChooseArea(GridGraphicSite* site);
    protected:
        virtual void paintEvent(QPainter& painter);
    };

    class GridColumnChooseWidget: public GridGraphicSite
    {
        Q_OBJECT
    public:
        friend class GridFieldChooserWindow;
        friend class GridColumnChooseArea;
        GridColumnChooseWidget(QWidget* parent, GridFieldChooserProvider* provider);
        virtual ~GridColumnChooseWidget();
        void highlightBackground(bool active);
    protected:
        virtual void resizeEvent(QResizeEvent *);
        virtual void paintEvent(QPaintEvent *);
        virtual void moveDragDropObject(const QPoint& pos, GridDragDropObject* object);
    private:
        void columnsUpdated();
        QVector<GridColumnElementGraphic *> m_colGroups;
        GridFieldChooserProvider* m_provider;
        bool m_highlightBackground;
    };

    class GridFieldChooserProvider: QObject
    {
    public:
        GridFieldChooserProvider(QObject* parent) : QObject(parent) {}
        virtual const QString& title() const = 0;
        virtual const QString& description() const = 0;
        virtual GridViewBase::GridColumns fields() const = 0;
        virtual GridViewBase* view() const = 0;
    };

    class GridFieldChooserWindow: public QWidget
    {
        Q_OBJECT
    public:
        GridFieldChooserWindow(GridViewBase* view);
        void clear();
        void addFiledProvider(GridFieldChooserProvider* provider);
        void startDragColumn(GridColumnBase* columnBase);
        void moveDragColumnOver();
        void moveDragColumnOut();
    protected:
        virtual void resizeEvent(QResizeEvent *);
    public slots:
        void columnsUpdated(QObject* sender);
    private:
        QTabWidget* m_tabs;
        QList<GridFieldChooserProvider *> m_providers;
    };

    class GridFloatToolWidget: public QWidget
    {
    public:
        GridFloatToolWidget();
    };

    class DragColumnWidget: public GridFloatToolWidget
    {
    public:
        DragColumnWidget(GridColumnBase* columnBase, GridGraphicPrimitive* gp);
    protected:
        virtual void paintEvent(QPaintEvent *);
    private:
        GridColumnBase* m_columnBase;
        GridGraphicPrimitive* m_gp;
    };

    class ColumnArrowWidget: public GridFloatToolWidget
    {
    public:
        ColumnArrowWidget(LayoutZone arrowDirection);
    protected:
        virtual void paintEvent(QPaintEvent *);
    private:
        QPixmap m_arrow;
    };

    enum ScrollBarsMode
    {
        ScrollNone,
        ScrollBoth,
        ScrollHorizontal,
        ScrollVertical,
        ScrollAuto
    };

    class QTITAN_EXPORT GridViewOptions: public QObject
    {
    public:
        GridViewOptions(GridViewBase* view);
        ScrollBarsMode scrollBars() const;
        void setScrollBars(ScrollBarsMode mode);
        bool groupsHeader() const;
        void setGroupsHeader(bool groupsHeader);
        GridScrollStyle scrollRowStyle() const;
        void setScrollRowStyle(GridScrollStyle style);
        bool autoScroll() const;
        void setAutoScroll(bool autoScroll);
        const QFont& columnFont() const;
        void setColumnFont(const QFont& font);
        const QPen& columnPen() const;
        void setColumnPen(const QPen& pen);
        bool isSortEnabled() const;
        void setSortEnabled(bool enabled);
        bool isGroupSortEnabled() const;
        void setGroupSortEnabled(bool enabled);
        bool modelDecoration() const;
        void setModelDecoration(bool enabled);
        bool rubberBandSelection() const;
        void setRubberBandSelection(bool enabled);
        int borderWidth() const;
        void setBorderWidth(int width);
        const QColor& borderColor() const;
        void setBorderColor(const QColor& color);
        const QBrush& groupsHeaderBrush() const;
        void setGroupsHeaderBrush(const QBrush& brush);
        const QColor& groupsHeaderTextColor() const;
        void setGroupsHeaderTextColor(const QColor& color);
        const QString& groupsHeaderText() const;
        void setGroupsHeaderText(const QString& text);
        const QColor& backgroundColor() const;
        void setBackgroundColor(const QColor& color);
        bool transparent() const;
        void setTransparent(bool t);
    protected:
        GridViewBase* view() const;
    private:
        GridViewBase* m_view;
        ScrollBarsMode m_scrollBarsMode;
        bool m_groupsHeader;
	    GridScrollStyle m_scrollRowStyle;
        bool m_autoScroll;
        QFont m_columnFont;
        QPen m_columnPen;
        bool m_isSortEnabled;
        bool m_isGroupSortEnabled;
        bool m_modelDecoration;
        bool m_rubberBandSelection;
        int m_borderWidth;
        QColor m_borderColor;
        QBrush m_groupsHeaderBrush;
        QColor m_groupsHeaderTextColor;
        QString m_groupsHeaderText;
        QColor m_backgroundColor;
        bool m_transparent;
    };

    int get_column_height_by_font1(QStyle* style, const QFont& font);
    int get_column_width_by_font(const QFont& font, const QString& text);

}; //namespace Qtitan

Q_DECLARE_OPERATORS_FOR_FLAGS(Qtitan::IconApperance)

#endif //QTN_GRID_H

