/****************************************************************************
**
** Qtitan Library by Developer Machines (Advanced Grid for Qt)
** 
** Copyright (c) 2009-2011 Developer Machines (http://www.devmachines.com)
**           ALL RIGHTS RESERVED
** 
**  The entire contents of this file is protected by copyright law and
**  international treaties. Unauthorized reproduction, reverse-engineering
**  and distribution of all or any portion of the code contained in this
**  file is strictly prohibited and may result in severe civil and 
**  criminal penalties and will be prosecuted to the maximum extent 
**  possible under the law.
**
**  RESTRICTIONS
**
**  THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED
**  FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE
**  COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE
**  AVAILABLE TO OTHER INDIVIDUALS WITHOUT WRITTEN CONSENT
**  AND PERMISSION FROM DEVELOPER MACHINES
**
**  CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON
**  ADDITIONAL RESTRICTIONS.
**
****************************************************************************/

#ifndef QTN_GRIDBANDEDTABLEVIEW_H
#define QTN_GRIDBANDEDTABLEVIEW_H

#include "QtnGrid.h"
#include "QtnGridTableView.h"

namespace Qtitan
{
    class GridBandedTableView;
    class GridTableBand;
    class GridBandedTableColumn;
    class GridBandedTableViewOptions;


    class GridBandedTableDragColumnObject: public GridTableDragColumnObject
    {
    public:
        GridBandedTableDragColumnObject(GridBandedTableView* view, const QPoint& point, GridTableColumnBase* column, GridGraphicPrimitive* graphic);
        virtual bool checkDropColumnTarget(const QPoint& pos, GridTableColumnBase* target);
        virtual void clear();
        int sourceBandIndex() const;
        void setSourceBandIndex(int bandIndex);
        int sourceRowIndex() const;
        void setSourceRowIndex(int rowIndex);
        int insertedRowIndex() const;
        void setInsertedRowIndex(int index);
        LayoutZone bandZone() const;
        void setBandZone(LayoutZone bandZone);
    protected:
        virtual void acceptEvent();
        virtual void calculateViewCoordinates(GridTableColumnBase* target, QPoint& viewPos, int& viewWidth) const;
        void showLeftRightArrows(GridTableColumnBase* target, LayoutZone zone);
    private:
        int m_sourceBandIndex;
        int m_sourceRowIndex;
        int m_insertedRowIndex;
        LayoutZone m_bandZone;
    };


    class GridBandedTableViewGraphic: public GridTableViewGraphic
    {
    public:
	    GridBandedTableViewGraphic(GridBandedTableView* view);
    protected:
        virtual void updateLayout();
    private:
        GridBandedTableView* m_view;
    };

    typedef QList<GridTableBand *> GridTableBands;
    typedef QList<GridBandedTableColumn *> GridBandedTableColumns;

    class QTITAN_EXPORT GridBandedTableView: public GridTableView
    {
        Q_OBJECT
    public:
        friend class GridBandedTableViewGraphic;
        friend class GridBandedTableColumn;
        friend class GridTableBand;
        friend class GridBandedTableDragColumnObject;
        friend class GridBandedTableViewOptions;
        friend class GridBandedTableHeaderGraphic;
        GridBandedTableColumn* addColumn(const QString& modelColumnName, const QString& caption = QString());
        GridBandedTableColumn* addColumn(int modelColumn, const QString& caption = QString());
        GridTableBand* addBand(const QString& caption);
        void removeBand(int index);
        int bandCount() const;
        GridTableBand* getBand(int index) const;
        static GridBandedTableView* create(Grid* grid);
    public:
        GridBandedTableView(Grid* grid);
        virtual ~GridBandedTableView();
        virtual void init();
        virtual bool hasVisibleParent(const GridColumnBase* column) const; 
        int columnRowCount(GridTableBand* band) const;
        void getBottomLevelBands(GridTableBand* band, GridTableBands& bands) const;
        GridTableBand* getTopLevelBand(const GridTableColumnBase* columnBase) const;
        void createColumnRow(int row, GridBandedTableColumn* column);
        void createBandRow(GridTableBand* band, int bandIndex, LayoutZone bandZone);
        GridBandedTableViewOptions& options() const;
        virtual void setModel(QAbstractItemModel* model, QModelIndex parent = QModelIndex(), bool createColumns = true);
    public:
        virtual void navigateUp(Qt::KeyboardModifiers modifiers = Qt::NoModifier);
        virtual void navigateDown(Qt::KeyboardModifiers modifiers = Qt::NoModifier);
        virtual GridTableColumn* getNextColumn(const GridTableColumn* anchor);
        virtual GridTableColumn* getPrevColumn(const GridTableColumn* anchor);
    protected:
#ifndef QTN_NOUSE_XML_MODULE
        virtual bool saveLayoutToStream(QXmlStreamWriter* xmlwriter);
        virtual bool loadLayoutFromStream(QXmlStreamReader* xmlreader);
        virtual bool saveTableColumnLayout(const GridTableColumn* column, QXmlStreamWriter* xmlwriter);
        virtual bool loadTableColumnLayout(GridTableColumn* column, QXmlStreamReader* xmlreader);
#endif
        virtual int getColumnRowIndex(const GridTableColumn* column) const;
        void setColumnRowIndex(GridBandedTableColumn* column, int rowIndex);
        virtual int getColumnRowSpan(const GridTableColumn* column) const;
        virtual void setColumnRowSpan(GridTableColumn* column, int rowSpan);
        void ensureRowValid(GridTableBand* band, int row);
        virtual int getVisualIndex(const GridTableColumnBase* columnBase) const;
        virtual void setVisualIndex(GridTableColumnBase* tableColumn, int visualIndex);
        virtual GridColumnBase* createColumn(int modelColumn, const QString& caption);
        virtual int getColumnIndex(const GridColumnBase* columnBase) const;
        virtual GridTableColumnBase* getPreviousResizeColumn(const GridTableColumnBase* column) const;
        virtual GridTableColumnBase* getNextResizeColumn(const GridTableColumnBase* column) const;
        virtual bool isVerticalResizable(const GridTableColumnBase* columnBase) const;
        virtual bool isHorizontalResizable(const GridTableColumnBase* columnBase) const;
        virtual ColumnFixedKind getColumnFixedKind(const GridTableColumnBase* columnBase) const;
        virtual void setColumnFixedKind(GridTableColumnBase* columnBase, ColumnFixedKind kind);
        virtual void getColumnLine(const GridColumnBase* column, GridColumns& retval) const;
        virtual int getMaximumVisualIndex(const GridTableColumnBase* anchor) const;
        virtual void columnWidthChanged(GridColumnBase* column);
        virtual int getColumnViewWidth(GridTableColumnBase* column);
        virtual int columnMinWidth(GridTableColumnBase* column);
        virtual int columnMaxWidth(GridTableColumnBase* column);
        virtual bool getColumnVisible(const GridColumnBase* columnBase) const;
        virtual void setColumnVisible(GridColumnBase* columnBase, bool visible);
        virtual GridViewOptions* createOptions();
        virtual GridHeaderPanelGraphic* createHeaderPanelGraphic();
        virtual void updateViewFixedMask();
        virtual void getFixedColumns(GridTableColumnBase*& left, GridTableColumnBase*& right) const;
        virtual bool checkRemove(const GridTableColumnBase* columnBase) const;
        virtual bool checkMove(const GridTableColumnBase* columnBase) const;
        virtual bool checkHorSizing(const GridTableColumnBase* columnBase) const;
        virtual bool checkVerSizing(const GridTableColumnBase* columnBase) const;
        virtual const QFont& getFont(const GridColumnBase* column) const;
        virtual const QPen& getPen(const GridColumnBase* column) const;
        int getBandIndex(const GridTableBand* band) const;
        void addNestedBand(GridTableBand* band);
        int removeNestedBand(GridTableBand* band);
        void parentBandChanged(GridTableBand* band);
        void parentBandChanged(GridBandedTableColumn* column);
        const GridTableBands& bands() const;
    protected:
	    virtual GridViewGraphic* createGraphic();
        virtual bool hasGroupingIndent(const GridTableColumnBase* column) const;
        virtual int getColumnLineCount() const;
        virtual int getColumnHeight(GridColumnBase* columnBase) const;
    private:
        void stretchBandToView(GridTableBand* ignoreBand);
        void collectBandChilds(GridTableBand* band, GridColumns& cols) const;
        GridTableBands m_bandslist;
        GridTableBands m_rootBands;
        GridFieldChooserProvider* m_bandProvider;
        int m_topLayouterRowCount;
    };

    class QTITAN_EXPORT GridTableBand: public GridTableColumnBase
    {
        Q_OBJECT
    public:
        friend class GridBandedTableView;
	    friend class GridBandedTableHeaderGraphic;
        friend class GridBandedTableDragColumnObject;
        friend class GridBandedTableViewOptions;
        GridTableBand(GridBandedTableView* view, const QString& caption);
        inline GridBandedTableView* view() const { return GridColumnBase::view<GridBandedTableView>(); }
        void addNestedColumn(GridBandedTableColumn* column);
        void removeNestedColumn(GridBandedTableColumn* column);
        void addNestedBand(GridTableBand* band);
        int removeNestedBand(GridTableBand* band);
        bool isRootBand() const;
        bool isBottomBand() const;
        bool isDescendant(GridTableBand* band) const;
        GridTableBand* parentBand() const;
        int bandIndex() const;
        void setBandIndex(int index);
        int levelCount() const;
        virtual GridBandedTableColumn* getNextColumn(const GridBandedTableColumn* anchor);
        virtual GridBandedTableColumn* getPrevColumn(const GridBandedTableColumn* anchor);
    protected:
        virtual void click(QMouseEvent* e);
        virtual GridTableDragColumnObject* createDragDropColumnObject(const QPoint& point, GridGraphicPrimitive* graphic);
        virtual bool needHeaderHints() const;
        virtual void initStyleOption(QStyleOptionHeader& styleOption);
        virtual void ensureWidth();
    private:
       void alignNestedColumns(GridBandedTableColumn* column = Q_NULL);
        void alignNestedBands(int visualIndex = -1);
        void updateFixedState();
        void moveColumnsToBottmLevel();
        GridBandedTableColumns m_nestedCols;
        GridTableBands m_nestedBands;
        GridTableBand* m_parentBand;
    };

    class QTITAN_EXPORT GridBandedTableColumn: public GridTableColumn
    {
        Q_OBJECT
        friend class GridBandedTableView;
        friend class GridTableBand;
        friend class GridBandedTableDragColumnObject;
        friend class GridBandedTableHeaderGraphic;
    public:
        GridBandedTableColumn(GridBandedTableView* view, const QString& caption);
        inline GridBandedTableView* view() const { return GridColumnBase::view<GridBandedTableView>(); }
        int bandIndex() const;
        void setBandIndex(int index);
        int rowIndex() const;
        void setRowIndex(int index);
        int rowSpan() const;
        void setRowSpan(int span);
        GridTableBand* parentBand() const;
        bool isVerSizingEnabled() const;
        void setVerSizingEnabled(bool enabled);
    protected:
        virtual GridTableDragColumnObject* createDragDropColumnObject(const QPoint& point, GridGraphicPrimitive* graphic);
    private:
        int m_row;
        int m_rowSpan;
        GridTableBand* m_parentBand;
        bool m_isVerSizingEnabled;
    };


    class QTITAN_EXPORT GridBandedTableViewOptions: public GridTableViewOptions
    {
    protected:
        friend class GridBandedTableView;
        GridBandedTableViewOptions(GridBandedTableView* view);
    public:
        bool bandsHeader() const;
        void setBandsHeader(bool bandsHeader);
        bool bandsQuickCustomization() const;
        void setBandsQuickCustomization(bool quickCustomization);
        int bandsQuickCustomizationMaxDropDownCount() const;
        void setBandsQuickCustomizationMaxDropDownCount(int dropDownCount);
        bool bandHeaderHints() const;
        void setBandHeaderHints(bool headerHints);
        bool isBandHidingEnabled() const;
        void setBandHidingEnabled(bool enabled);
        bool isBandMovingEnabled() const;
        void setBandMovingEnabled(bool enabled);
        bool changeNestedBandOnMoving() const;
        void setChangeNestedBandOnMoving(bool changeNestedBandOnMoving);
        bool changeColumnRowOnMoving() const;
        void setChangeColumnRowOnMoving(bool changeColumnRowOnMoving);
        bool isColumnVerSizingEnabled() const;
        void setColumnVerSizingEnabled(bool enabled);
        bool isBandHorSizingEnabled() const;
        void setBandHorSizingEnabled(bool enabled);
        const QFont& bandFont() const;
        void setBandFont(const QFont& font);
        const QPen& bandPen() const;
        void setBandPen(const QPen& pen);
        int bandHeight() const;
        void setBandHeight(int height);
    private:
        bool m_bandsHeader;
        bool m_bandsQuickCustomization;
        bool m_bandHeaderHints;
        int m_bandsDropDownCount;
        bool m_isBandHidingEnabled;
        bool m_isBandMovingEnabled;
        bool m_changeNestedBandOnMoving;
        bool m_changeColumnRowOnMoving;
        bool m_isColumnVerSizingEnabled;
        bool m_isBandHorSizingEnabled;
        QPen m_bandPen;
        QFont m_bandFont;
        int m_bandHeight;
    };

}; //namespace Qtitan

#endif //QTN_GRIDBANDEDTABLEVIEW_H

