/****************************************************************************
**
** Qtitan Library by Developer Machines (Advanced Grid for Qt)
** 
** Copyright (c) 2009-2011 Developer Machines (http://www.devmachines.com)
**           ALL RIGHTS RESERVED
** 
**  The entire contents of this file is protected by copyright law and
**  international treaties. Unauthorized reproduction, reverse-engineering
**  and distribution of all or any portion of the code contained in this
**  file is strictly prohibited and may result in severe civil and 
**  criminal penalties and will be prosecuted to the maximum extent 
**  possible under the law.
**
**  RESTRICTIONS
**
**  THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED
**  FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE
**  COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE
**  AVAILABLE TO OTHER INDIVIDUALS WITHOUT WRITTEN CONSENT
**  AND PERMISSION FROM DEVELOPER MACHINES
**
**  CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON
**  ADDITIONAL RESTRICTIONS.
**
****************************************************************************/

#ifndef QTN_GRID_EDITORS_H
#define QTN_GRID_EDITORS_H

#include <QtCore/QHash>
#include <QtGui/QTextEdit>
#include <QtGui/QComboBox>
#include <QtGui/QLineEdit>
#include <QtGui/QDoubleSpinBox>
#include <QtGui/QCheckBox>
#include <QtGui/QDateTimeEdit>
#include <QtGui/QLabel>
#include <QtGui/QSizeGrip>
#include <QtGui/QScrollArea>
#include <QtGui/QPushButton>
#include <QtGui/QToolButton>
#include <QtGui/QPrinter>

#include "QtnGrid.h"

namespace Qtitan
{
    class GridEditor;
    class GridEditorRepository;
    class PictureEditorWindow;
    class PictureComboBox;
    class PictureEditorWidget;
    class TextComboBox;
    class TextEditorWidget;

    class GridEditorSite
    {
    public:
        virtual void repaint() = 0;
        virtual void update(bool fullyViewUpdate = false) = 0;
        virtual QPoint globalPos() const = 0;
        virtual ModelDataBinding* dataBinding() const = 0;
        virtual int columnIndex() const = 0;
    };


    class QTITAN_EXPORT EditorEventArgs: public BaseEventArgs
    {
    public:
        EditorEventArgs(GridColumnBase* column, GridEditor* e);
        GridColumnBase* column() const;
        GridEditor* editor() const;
        ModelDataBinding* dataBinding() const;
    private:
        GridColumnBase* m_column;
        GridEditor* m_e;
    };

    enum GridEditorType
    {
        AutoEditorType,
        StringEditorType,
        MemoEditorType,
        ComboBoxBlobEditorType,
        NumericEditorType,
        CheckBoxEditorType,
        ComboBoxEditorType,
        TableEditorType,
        DateTimeEditorType,
        TimeEditorType,
        DateEditorType,
        PictureEditorType,
        ComboBoxPictureEditorType,
        DelegateAdapterEditorType,
        CustomEditType
    };

    class QTITAN_EXPORT GridEditorStyleOption: public QStyleOption
    {
    public:
        GridEditorStyleOption(int version = QStyleOption::Version, int type = SO_Default);
    public:
        QFont font;
        QIcon icon;
        QSize iconSize;
    };

    class QTITAN_EXPORT GridEditor: public QObject
    {
    public:
        GridEditor(GridEditorRepository* editorRepository, GridEditorSite* editorSite);
        virtual ~GridEditor();
        GridEditorRepository* editorRepository() const;
        GridEditorSite* site() const;
        virtual const QRect& rect() const;
        virtual void setRect(const QRect& r);
        virtual QWidget* displayWidget();
        virtual QWidget* editWidget();
        virtual void paintEvent(QPainter& painter, const GridEditorStyleOption& option);
        virtual void mouseMoveEvent(const QPoint& pos);
        virtual void mousePressEvent(const QPoint& pos);
        virtual void mouseReleaseEvent(const QPoint& pos);
        virtual void mouseDblClickEvent(const QPoint& pos);
        virtual void mouseOverEvent(const QPoint& pos);
        virtual void mouseOutEvent();
        virtual void setDisplayValue(const QVariant& v);
        virtual QVariant displayValue() const;
        virtual QVariant editValue() const = 0;
        virtual void startEdit(const QVariant& editValue, bool modified) = 0;
        virtual void stopEdit() = 0;
        virtual bool isEditing() const = 0;
        virtual bool isModified() const = 0;
        virtual void updateSettings();
    protected:
        void raiseEditingStarted();
        void raiseEditingStoping();
        GridEditorRepository* m_editorRepository;
    private:
        GridEditorSite* m_site;
        QRect m_rect;
        QVariant m_displayValue;
        bool m_needToolTip;
    };

    enum GridEditorFocusPolicyFlag
    {
        NoPolicy = 0x0000,
        ClickNoFocused = 0x0001,
        ClickFocused = 0x0002,
        Click = ClickNoFocused | ClickFocused,
        MousePressNoFocused = 0x0004,
        MousePressFocused = 0x0008,
        MousePress = MousePressNoFocused | MousePressFocused,
        DblClickNoFocused = 0x0010,
        DblClickFocused = 0x0020,
        DblClick = DblClickNoFocused | DblClickFocused,
        KeyboardFocus = 0x0040,
        Enter = 0x0080,
        Always = 0x0100
    };

    Q_DECLARE_FLAGS(GridEditorFocusPolicy, GridEditorFocusPolicyFlag)

    class QTITAN_EXPORT GridEditorRepository: public QObject
    {
        Q_OBJECT
    public:
        friend class GridEditor;
        GridEditorRepository(GridViewBase* view);
        virtual ~GridEditorRepository();
        virtual GridEditor* createEditor(GridEditorSite* site) = 0;
        void setAlignment(Qt::Alignment flag);
        Qt::Alignment alignment() const;
        const QIcon& icon() const;
        void setIcon(const QIcon& icon);
        IconApperance iconApperance() const;
        void setIconApperance(IconApperance apperance);
        bool isEnabled() const;
        void setEnabled(bool enabled);
        bool isEditable() const;
        void setEditable(bool editable);
        bool transparentBackground() const;
        void setTransparentBackground(bool transparent);
        GridEditorFocusPolicy editorFocusPolicy() const;
        void setEditorFocusPolicy(GridEditorFocusPolicy policy);
        bool alwaysShowEditor() const;
        void setAlwaysShowEditor(bool flag);
        bool autoSelect() const;
        void setAutoSelect(bool autoSelect);
        bool isHintVisible() const;
        void setHintVisible(bool visible);
        bool validateOnEnter() const;
        void setValidateOnEnter(bool flag);
        virtual bool canValidate();
        virtual bool validate(QVariant& editValue, int& errorCode, QString& errorReason);
        QValidator* validator() const;
        void setValidator(QValidator* validator);
        virtual int heightHint(int width, const QVariant& v);
    public:
        inline GridViewBase* view() const { return m_view; }
    Q_SIGNALS:
        void validateValue(QVariant& editValue, int& errorCode, QString& errorReason);
    protected:
        void update();
        GridViewBase* m_view;
    private:
        void addEditor(GridEditor* editor);
        void removeEditor(GridEditor* editor);
        QList<GridEditor *> m_editors;
        Qt::Alignment m_alignment;
        QIcon m_icon;
        IconApperance m_iconApperance;
        bool m_enabled;
        bool m_editable;
        bool m_transparentBackground;
        GridEditorFocusPolicy m_editorFocusPolicy;
        bool m_alwaysShowEditor;
        bool m_autoSelect;
        bool m_hintVisible;
        bool m_validateOnEnter;
        QValidator* m_externalValidator;
        QValidator* m_validator;
    };

    class QTITAN_EXPORT GridEditorRepositoryBase: public GridEditorRepository
    {
    public:
        GridEditorRepositoryBase(GridViewBase* view)
            : GridEditorRepository(view)
        {}
    protected:
    };

    class QTITAN_EXPORT GridStringEditorRepository: public GridEditorRepositoryBase
    {
    public:
        GridStringEditorRepository(GridViewBase* view);
        virtual GridEditor* createEditor(GridEditorSite* site);
    public:
        const QString& inputMask() const;
        void setInputMask(const QString& mask);
        QLineEdit::EchoMode echoMode() const;
        void setEchoMode(QLineEdit::EchoMode mode);
        int maxLength() const;
        void setMaxLength(int length);
    private:
        QString m_inputMask;
        QLineEdit::EchoMode m_echoMode;
        int m_maxLength;
    };

    class QTITAN_EXPORT GridStringEditor: public GridEditor
    {
        Q_OBJECT
    public:
        GridStringEditor(GridEditorRepository* editor, GridEditorSite* editorSite);
        virtual ~GridStringEditor();
        virtual QWidget* displayWidget();
        virtual QWidget* editWidget();
        virtual QVariant editValue() const;
        virtual void startEdit(const QVariant& editValue, bool modified);
        virtual void stopEdit();
        virtual bool isEditing() const;
        virtual bool isModified() const;
        virtual void updateSettings();
    private:
        QLineEdit* m_textEdit;
        bool m_modified;
    private slots:
        void textChanged(const QString &text);
    };

    class QTITAN_EXPORT GridNumericEditorRepository: public GridEditorRepositoryBase
    {
    public:
        GridNumericEditorRepository(GridViewBase* view);
        virtual GridEditor* createEditor(GridEditorSite* site);
    public:
        double maximum() const;
        void setMaximum(double max);
        double minimum() const;
        void setMinimum(double min);
        void getRange(double& minimum, double& maximum) const;
        void setRange(double minimum, double maximum);
        const QString& prefix() const;
        void setPrefix(const QString& prefix);
        const QString& suffix() const;
        void setSuffix(const QString& suffix);
        int decimals() const;
        void setDecimals(int prec);
        void setSingleStep(double val);
        double singleStep() const;
    private:
        double m_maximum;
        double m_minimum;
        QString m_prefix;
        QString m_suffix;
        int m_decimals;
        double m_singleStep;
    };

    class QTITAN_EXPORT GridNumericEditor: public GridEditor
    {
        Q_OBJECT
    public:
        GridNumericEditor(GridEditorRepository* editor, GridEditorSite* editorSite);
        virtual ~GridNumericEditor();
        virtual QWidget* displayWidget();
        virtual QWidget* editWidget();
        virtual QVariant editValue() const;
        virtual void startEdit(const QVariant& editValue, bool modified);
        virtual void stopEdit();
        virtual bool isEditing() const;
        virtual bool isModified() const;
        virtual void updateSettings();
    private:
        QDoubleSpinBox* m_numericEdit;
        bool m_modified;
    private slots:
        void valueChanged(double d);
    };

    class QTITAN_EXPORT GridCheckBoxEditorRepository: public GridEditorRepositoryBase
    {
        friend class GridCheckBoxEditor;
    public:
        GridCheckBoxEditorRepository(GridViewBase* view);
        virtual GridEditor* createEditor(GridEditorSite* site);
        void setTristate(bool state = true);
        bool isTristate() const;
    private:
        bool m_tristate;
    };

    class QTITAN_EXPORT CheckEdit: public QWidget
    {
    public:
        CheckEdit(QWidget* parent = 0);
        QCheckBox* box() const;
        void setAlignment(Qt::Alignment flag);
        Qt::Alignment alignment() const;
    protected:
        virtual void resizeEvent(QResizeEvent *);
        virtual bool event(QEvent *);
    private:
        void updateAlign();
        QCheckBox* m_cb;
        Qt::Alignment m_alignment;
    };

    class QTITAN_EXPORT GridCheckBoxEditor: public GridEditor
    {
        Q_OBJECT
    public:
        GridCheckBoxEditor(GridEditorRepository* editor, GridEditorSite* editorSite);
        virtual ~GridCheckBoxEditor();
        virtual QWidget* displayWidget();
        virtual QWidget* editWidget();
        virtual QVariant editValue() const;
        virtual void startEdit(const QVariant& editValue, bool modified);
        virtual void stopEdit();
        virtual bool isEditing() const;
        virtual bool isModified() const;
        virtual void updateSettings();
    private:
        void setValueToCheckBox(const QVariant& value);
        CheckEdit* m_checkEdit;
        bool m_modified;
    private slots:
        void stateChanged(int state);
    };

    class QTITAN_EXPORT GridComboBoxEditorRepository: public GridEditorRepositoryBase
    {
        friend class GridComboBoxEditor;
    public:
        GridComboBoxEditorRepository(GridViewBase* view);
        virtual GridEditor* createEditor(GridEditorSite* site);
    private:
    };

    class QTITAN_EXPORT GridComboBoxEditor: public GridEditor
    {
        Q_OBJECT
    public:
        GridComboBoxEditor(GridEditorRepository* editor, GridEditorSite* editorSite);
        virtual ~GridComboBoxEditor();
        virtual QWidget* displayWidget();
        virtual QWidget* editWidget();
        virtual void paintEvent(QPainter& painter, const GridEditorStyleOption& option);
        virtual QVariant editValue() const;
        virtual void startEdit(const QVariant& editValue, bool modified);
        virtual void stopEdit();
        virtual bool isEditing() const;
        virtual bool isModified() const;
        virtual void updateSettings();
    private:
        void setValueTo(const QVariant& value);
        QComboBox* m_comboBox;
        bool m_modified;
    private slots:
        void currentIndexChanged(int index);
    };

    class QTITAN_EXPORT GridDateTimeEditorRepository: public GridEditorRepositoryBase
    {
    public:
        GridDateTimeEditorRepository(GridViewBase* view);
        virtual GridEditor* createEditor(GridEditorSite* site);
        bool calendarPopup() const;
        void setCalendarPopup(bool flag);
        QDateTime maximumDateTime() const;
        void setMaximumDateTime(const QDateTime& dt);
        QDate maximumDate() const;
        void setMaximumDate(const QDate& max);
        QTime maximumTime() const;
        void setMaximumTime(const QTime& max);
        QDate minimumDate() const;
        void setMinimumDate(const QDate& min);
        QDateTime minimumDateTime() const;
        void setMinimumDateTime(const QDateTime& dt);
        QTime minimumTime() const;
        void setMinimumTime(const QTime& min);
        Qt::TimeSpec timeSpec() const;
        void setTimeSpec(Qt::TimeSpec spec);
        const QString& displayFormat() const;
        void setDisplayFormat(const QString& format);
        void getDateRange(QDate &min, QDate& max) const;
        void setDateRange(const QDate& min, const QDate& max);
        void getDateTimeRange(QDateTime& min, QDateTime& max) const;
        void setDateTimeRange(const QDateTime& min, const QDateTime& max);
        void getTimeRange(QTime& min, QTime& max) const;
        void setTimeRange(const QTime& min, const QTime& max);
    private:
        bool m_calendarPopup;
        QDateTime m_maximumDateTime;
        QDate m_maximumDate;
        QTime m_maximumTime;
        QDate m_minimumDate;
        QDateTime m_minimumDateTime;
        QTime m_minimumTime;
        Qt::TimeSpec m_timeSpec;
        QString m_displayFormat;
    };

    class QTITAN_EXPORT GridDateTimeEditor: public GridEditor
    {
        Q_OBJECT
    public:
        GridDateTimeEditor(GridEditorRepository* editor, GridEditorSite* editorSite);
        virtual ~GridDateTimeEditor();
        virtual QWidget* displayWidget();
        virtual QWidget* editWidget();
        virtual QVariant editValue() const;
        virtual void startEdit(const QVariant& editValue, bool modified);
        virtual void stopEdit();
        virtual bool isEditing() const;
        virtual bool isModified() const;
        virtual void updateSettings();
    protected:
        virtual QDateTimeEdit* createDateTimeWidget();
    private:
        void setValueToDT(const QVariant& value);
        QDateTimeEdit* m_dtEdit;
        bool m_modified;
    private slots:
        void dateTimeChanged(const QDateTime& datetime);
    };

    class QTITAN_EXPORT GridTimeEditorRepository: public GridDateTimeEditorRepository
    {
    public:
        GridTimeEditorRepository(GridViewBase* view);
        virtual GridEditor* createEditor(GridEditorSite* site);
    private:
    };

    class QTITAN_EXPORT GridTimeEditor: public GridDateTimeEditor
    {
    public:
        GridTimeEditor(GridEditorRepository* editor, GridEditorSite* editorSite);
    protected:
        virtual QDateTimeEdit* createDateTimeWidget();
    private:
    };

    class QTITAN_EXPORT GridDateEditorRepository: public GridDateTimeEditorRepository
    {
    public:
        GridDateEditorRepository(GridViewBase* view);
        virtual GridEditor* createEditor(GridEditorSite* site);
    private:
    };

    class QTITAN_EXPORT GridDateEditor: public GridDateTimeEditor
    {
    public:
        GridDateEditor(GridEditorRepository* editor, GridEditorSite* editorSite);
    protected:
        virtual QDateTimeEdit* createDateTimeWidget();
    private:
    };

    class QTITAN_EXPORT GridPictureEditorRepository: public GridEditorRepositoryBase
    {
        Q_OBJECT
    public:
        friend class GridPictureEditor;
        GridPictureEditorRepository(GridViewBase* view);
        virtual GridEditor* createEditor(GridEditorSite* site);
        Qt::TransformationMode transformMode() const;
        void setTransformMode(Qt::TransformationMode mode);
        Qt::AspectRatioMode aspectRatioMode() const;
        void setAspectRatioMode(Qt::AspectRatioMode mode);
        bool isScaled() const;
        void setScaled(bool scaled);
        virtual int heightHint(int width, const QVariant& v);
        const QString& imageEmptyText() const;
        void setImageEmptyText(const QString& text);
    private:
        Qt::TransformationMode m_transformMode;
        Qt::AspectRatioMode m_aspectRatioMode;
        bool m_scaled;
        QString m_imageEmptyText;
    private slots:
        void hidePictureEditor();
    };

    class QTITAN_EXPORT GridPictureEditor: public GridEditor
    {
        Q_OBJECT
    public:
        GridPictureEditor(GridEditorRepository* editor, GridEditorSite* editorSite);
        virtual ~GridPictureEditor();
        virtual void setRect(const QRect& r);
        virtual QWidget* displayWidget();
        virtual QWidget* editWidget();
        virtual void paintEvent(QPainter& painter, const GridEditorStyleOption& option);
        virtual void setDisplayValue(const QVariant& v);
        virtual QVariant editValue() const;
        virtual void startEdit(const QVariant& editValue, bool modified);
        virtual void stopEdit();
        virtual bool isEditing() const;
        virtual bool isModified() const;
        virtual void updateSettings();
    private:
        PictureEditorWidget* m_inplaceWidget;
        QPixmap m_pixmap;
        QVariant::Type m_editType;
        bool m_modified;
    private slots:
        void pictureChanged();
    };

    class CustomComboBoxPopup;

    class QTITAN_EXPORT CustomComboBox: public QWidget
    {
        Q_OBJECT
    public:
        friend class CustomComboBoxPopup;
        CustomComboBox(QWidget* parent);
        virtual ~CustomComboBox();
        bool isModified() const;
        void setModified(bool modified);
        void setPopupOffset(const QPoint& pos);
        virtual void showPopup();
        virtual void hidePopup() = 0;
        bool okButtonVisible();
        void setOkButtonVisible(bool visible);
        bool cancelButtonVisible() const;
        void setCancelButtonVisible(bool visible) const;
        bool crossButtonVisible() const;
        void setCrossButtonVisible(bool visible);
    protected:
        virtual QWidget* editorWidget() = 0;
        virtual void undoModified() = 0;
        virtual void saveDefaultState();
        QStyle::SubControl newHoverControl(const QPoint &pos);
        bool updateHoverControl(const QPoint &pos);
        void updateArrow(QStyle::StateFlag state);
        bool event(QEvent *event);
        void initStyleOption(QStyleOptionComboBox *option) const;
        virtual void paintEvent(QPaintEvent* e);
        virtual void mousePressEvent(QMouseEvent *e);
        virtual void mouseReleaseEvent(QMouseEvent *e);
        virtual void keyPressEvent(QKeyEvent *e);
        virtual QPixmap getIcon() = 0;
        void updatePopupButtons();
    Q_SIGNALS:
        void dataChanged();
    private:
        QStyle::StateFlag arrowState;
        QStyle::SubControl hoverControl;
        QRect hoverRect;
    protected:
        CustomComboBoxPopup* m_popup;
        QIcon m_icon;
        QPoint m_popupOffset;
        bool m_modified;
    };

    class QTITAN_EXPORT CustomComboBoxPopup: public QWidget
    {
        Q_OBJECT
    public:
        CustomComboBoxPopup(CustomComboBox* parent, bool okButton = true, bool cancelButton = true, bool crossButton = true);
        virtual void init();
        void setAutoHiding(bool flag);
        QAbstractButton* okButton() const;
        QAbstractButton* cancelButton() const;
        QAbstractButton* crossButton() const;
    protected:
        virtual void hideEvent(QHideEvent *);
        virtual void paintEvent(QPaintEvent *);
        QWidget* viewWidget() const;
        virtual void setVisible(bool visible);
        inline CustomComboBox* comboBox() const { return m_comboBox; };
    private:
        QToolButton* m_okButton;
        QToolButton* m_cancelButton;
        QToolButton* m_closeButton;
        bool m_autoHiding;
        QWidget* m_viewWidget;
        QSizeGrip* m_grip;
        CustomComboBox* m_comboBox;
    public slots:
        void ok();
        void cancel();
        void doClose();
    };

    class QTITAN_EXPORT PictureEditorPopup: public CustomComboBoxPopup
    {
    public:
        PictureEditorPopup(PictureComboBox* parent);
    protected:
    };

    class QTITAN_EXPORT PictureComboBox: public CustomComboBox
    {
        Q_OBJECT
    public:
        PictureComboBox(QWidget* parent = Q_NULL);
        void setPixmap(const QPixmap& pm);
        const QPixmap& pixmap() const;
        virtual void showPopup();
        virtual void hidePopup();
    protected:
        virtual QPixmap getIcon();
        virtual QWidget* editorWidget();
        virtual void undoModified();
        virtual void saveDefaultState();
    private:
        PictureEditorWidget* m_pictureEditor;
        QPixmap m_statepm;
    protected slots:
        void pictureChanged();
    };

    class QTITAN_EXPORT PictureEditorWidget: public QScrollArea
    {
        Q_OBJECT
    public:
        PictureEditorWidget(QWidget* parent = Q_NULL);
        void setPixmap(const QPixmap& pixmap);
        const QPixmap* pixmap() const;
        bool isModified() const;
        void setModified(bool m = true);
    protected:
        virtual void create();
        virtual void contextMenuEvent(QContextMenuEvent* event);
        void setPixmapInternal(const QPixmap& pixmap, bool modified);
    private:
        void createActions();
        void zoomImage(double factor);
        void loadImage(const QString& fileName);
        void saveImage(const QString& fileName, const QString& format);
        QLabel* m_image;
        QAction* m_loadAction;
        QAction* m_saveAction;
        QAction* m_copyAction;
        QAction* m_pasteAction;
        QAction* m_zoomInAction;
        QAction* m_zoomOutAction;
        QAction* m_adjustSizeAction;
        QAction* m_actualSizeAction;
        QAction* m_printAction;
        QMenu* m_contextMenu;
        double m_zoomFactor;
        bool m_modified;
        QPrinter m_printer;
    public slots:
        void load();
        void save();
        void copy();
        void paste();
        void zoomIn();
        void zoomOut();
        void adjustSize();
        void actualSize();
        void print();
    Q_SIGNALS:
        void pictureChanged();
    };

    class QTITAN_EXPORT TableComboBox: public CustomComboBox
    {
        Q_OBJECT
    public:
        TableComboBox(QWidget* parent = Q_NULL);
        virtual void showPopup();
        virtual void hidePopup();
        Grid* grid() const;
        void setModel(QAbstractItemModel* model);
    protected:
        virtual QPixmap getIcon();
        virtual QWidget* editorWidget();
        virtual void undoModified();
        virtual void saveDefaultState();
    private:
        Grid* m_grid;
        int m_oldRowIndex;
    private slots:
        void focusRowChanged(QObject* sender, int oldRowIndex, int rowIndex);
    };

    class QTITAN_EXPORT TableComboBoxPopup: public CustomComboBoxPopup
    {
    public:
        TableComboBoxPopup(TableComboBox* parent);
    protected:
    };

    class QTITAN_EXPORT GridPictureComboBoxEditorRepository: public GridEditorRepositoryBase
    {
        Q_OBJECT
    public:
        friend class GridPictureComboBoxEditor;
        GridPictureComboBoxEditorRepository(GridViewBase* view);
        virtual GridEditor* createEditor(GridEditorSite* site);
        Qt::TransformationMode transformMode() const;
        void setTransformMode(Qt::TransformationMode mode);
        Qt::AspectRatioMode aspectRatioMode() const;
        void setAspectRatioMode(Qt::AspectRatioMode mode);
    private:
        Qt::TransformationMode m_transformMode;
        Qt::AspectRatioMode m_aspectRatioMode;
    private slots:
        void hidePictureEditor();
    };

    class QTITAN_EXPORT GridPictureComboBoxEditor: public GridEditor
    {
        Q_OBJECT
    public:
        GridPictureComboBoxEditor(GridEditorRepository* editor, GridEditorSite* editorSite);
        virtual ~GridPictureComboBoxEditor();
        virtual QWidget* displayWidget();
        virtual QWidget* editWidget();
        virtual void paintEvent(QPainter& painter, const GridEditorStyleOption& option);
        virtual void setDisplayValue(const QVariant& v);
        virtual QVariant editValue() const;
        virtual void startEdit(const QVariant& editValue, bool modified);
        virtual void stopEdit();
        virtual bool isEditing() const;
        virtual bool isModified() const;
        virtual void updateSettings();
    private:
        PictureComboBox* m_pictureComboBox;
        QVariant::Type m_editType;
        bool m_modified;
    private slots:
        void pictureChanged();
    };


    class QTITAN_EXPORT GridMemoEditorRepository: public GridEditorRepositoryBase
    {
        Q_OBJECT
    public:
        friend class GridMemoEditor;
        GridMemoEditorRepository(GridViewBase* view);
        virtual GridEditor* createEditor(GridEditorSite* site);
        virtual int heightHint(int width, const QVariant& v);
        void setCurrentCharFormat(const QTextCharFormat &format);
        QTextCharFormat currentCharFormat() const;
        QTextEdit::AutoFormatting autoFormatting() const;
        void setAutoFormatting(QTextEdit::AutoFormatting features);
        bool tabChangesFocus() const;
        void setTabChangesFocus(bool b);
        void setDocumentTitle(const QString &title);
        const QString& documentTitle() const;
        bool isUndoRedoEnabled() const;
        void setUndoRedoEnabled(bool enable);
        QTextEdit::LineWrapMode lineWrapMode() const;
        void setLineWrapMode(QTextEdit::LineWrapMode mode);
        int lineWrapColumnOrWidth() const;
        void setLineWrapColumnOrWidth(int w);
        QTextOption::WrapMode wordWrapMode() const;
        void setWordWrapMode(QTextOption::WrapMode policy);
        bool overwriteMode() const;
        void setOverwriteMode(bool overwrite);
        int tabStopWidth() const;
        void setTabStopWidth(int width);
        int cursorWidth() const;
        void setCursorWidth(int width);
        bool acceptRichText() const;
        void setAcceptRichText(bool accept);
        bool isHTML() const;
        void setHTML(bool html);
    protected:
        void setProperties(QTextEdit* edit) const;
    private:
        QTextCharFormat m_currentCharFormat;
        QTextEdit::AutoFormatting m_autoFormatting;
        bool m_tabChangesFocus;
        QString m_documentTitle;
        bool m_isUndoRedoEnabled;
        QTextEdit::LineWrapMode m_lineWrapMode;
        int m_lineWrapColumnOrWidth;
        QTextOption::WrapMode m_wordWrapMode;
        bool m_overwriteMode;
        int m_tabStopWidth;
        int m_cursorWidth;
        bool m_acceptRichText;
        bool m_isHTML;
    };

    class QTITAN_EXPORT GridMemoEditor: public GridEditor
    {
        Q_OBJECT
    public:
        GridMemoEditor(GridEditorRepository* editor, GridEditorSite* editorSite);
        virtual ~GridMemoEditor();
        virtual QWidget* displayWidget();
        virtual QWidget* editWidget();
        virtual void paintEvent(QPainter& painter, const GridEditorStyleOption& option);
        virtual QVariant editValue() const;
        virtual void startEdit(const QVariant& editValue, bool modified);
        virtual void stopEdit();
        virtual bool isEditing() const;
        virtual bool isModified() const;
        virtual void updateSettings();
    private:
        void updateRect();
        QTextEdit* m_inplaceWidget;
        bool m_modified;
    private slots:
        void textChanged();
    };

    class QTITAN_EXPORT TextComboBox: public CustomComboBox
    {
        Q_OBJECT
    public:
        friend class TextComboBoxPopup;
        TextComboBox(QWidget* parent = Q_NULL);
        QString text() const;
        void setText(const QString& text);
        bool isHTML() const;
        void setHTML(bool html);
        QTextEdit* textEdit();
        virtual void showPopup();
        virtual void hidePopup();
    protected:
        virtual QPixmap getIcon();
        virtual QWidget* editorWidget();
        virtual void undoModified();
        virtual void saveDefaultState();
    private:
        void createPopup();
        QTextEdit* m_textEditor;
        bool m_isHTML;
        QString m_statetext;
    private slots:
        void textChanged();
    };

    class QTITAN_EXPORT TextComboBoxPopup: public CustomComboBoxPopup
    {
        Q_OBJECT
    public:
        friend class TextComboBox;
        TextComboBoxPopup(TextComboBox* parent);
    private:
    };

    class QTITAN_EXPORT GridBlobEditorRepository: public GridMemoEditorRepository
    {
        Q_OBJECT
    public:
        friend class GridBlobEditor;
        GridBlobEditorRepository(GridViewBase* view);
        virtual GridEditor* createEditor(GridEditorSite* site);
    private:
    };

    class QTITAN_EXPORT GridBlobEditor: public GridEditor
    {
        Q_OBJECT
    public:
        GridBlobEditor(GridEditorRepository* editor, GridEditorSite* editorSite);
        virtual ~GridBlobEditor();
        virtual QWidget* displayWidget();
        virtual QWidget* editWidget();
        virtual void paintEvent(QPainter& painter, const GridEditorStyleOption& option);
        virtual QVariant editValue() const;
        virtual void startEdit(const QVariant& editValue, bool modified);
        virtual void stopEdit();
        virtual bool isEditing() const;
        virtual bool isModified() const;
        virtual void updateSettings();
    private:
        void updateRect();
        TextComboBox* m_textComboBox;
        bool m_modified;
    private slots:
        void textChanged();
    };

    class GridDelegateAdapterEditor;
    class QTITAN_EXPORT GridDelegateAdapterEditorRepository: public GridEditorRepositoryBase
    {
        Q_OBJECT
    public:
        friend class GridDelegateAdapterEditor;
        GridDelegateAdapterEditorRepository(GridViewBase* view);
        virtual GridEditor* createEditor(GridEditorSite* site);
    public:
        void setDelegate(QAbstractItemDelegate* itemDelegate);
        QAbstractItemDelegate* getDelegate() const;
    private:
        QAbstractItemDelegate* m_delegate;
        QHash<QWidget *, GridDelegateAdapterEditor *> m_delegateEditorHash;
    private slots:
        void delegateCloseEditor(QWidget* editor, QAbstractItemDelegate::EndEditHint hint = QAbstractItemDelegate::NoHint);
        void delegateCommitData(QWidget* editor);
        void delegateSizeHintChanged(const QModelIndex& index);
    };

    class QTITAN_EXPORT GridDelegateAdapterEditor: public GridEditor
    {
        Q_OBJECT
    public:
        friend class GridDelegateAdapterEditorRepository;
        GridDelegateAdapterEditor(GridEditorRepository* editor, GridEditorSite* editorSite);
        virtual ~GridDelegateAdapterEditor();
        virtual void setDisplayValue(const QVariant& v);
        virtual QVariant displayValue() const;
        virtual QWidget* displayWidget();
        virtual QWidget* editWidget();
        virtual void paintEvent(QPainter& painter, const GridEditorStyleOption& option);
        virtual QVariant editValue() const;
        virtual void startEdit(const QVariant& editValue, bool modified);
        virtual void stopEdit();
        virtual bool isEditing() const;
        virtual bool isModified() const;
        virtual void updateSettings();
    private:
        QWidget* m_widget;
        bool m_modified;
        QVariant m_oldEditValue;
        QAbstractItemModel* m_delegateModel;
    };

    class QTITAN_EXPORT GridTableEditorRepository: public GridEditorRepositoryBase
    {
        friend class GridTableEditor;
    public:
        GridTableEditorRepository(GridViewBase* view);
        virtual GridEditor* createEditor(GridEditorSite* site);
    private:
    };

    class QTITAN_EXPORT GridTableEditor: public GridEditor
    {
        Q_OBJECT
    public:
        GridTableEditor(GridEditorRepository* editor, GridEditorSite* editorSite);
        virtual ~GridTableEditor();
        virtual QWidget* displayWidget();
        virtual QWidget* editWidget();
        virtual void paintEvent(QPainter& painter, const GridEditorStyleOption& option);
        virtual QVariant editValue() const;
        virtual void startEdit(const QVariant& editValue, bool modified);
        virtual void stopEdit();
        virtual bool isEditing() const;
        virtual bool isModified() const;
        virtual void updateSettings();
    private:
        void setValueTo(const QVariant& value);
        TableComboBox* m_tableComboBox;
        bool m_modified;
        QVariant m_masterModeValue;
    private slots:
        void focusChanged();
    };

}; //namespace Qtitan


#endif //QTN_GRID_EDITORS_H


