/****************************************************************************
**
** Qtitan Library by Developer Machines (Advanced Grid for Qt)
** 
** Copyright (c) 2009-2011 Developer Machines (http://www.devmachines.com)
**           ALL RIGHTS RESERVED
** 
**  The entire contents of this file is protected by copyright law and
**  international treaties. Unauthorized reproduction, reverse-engineering
**  and distribution of all or any portion of the code contained in this
**  file is strictly prohibited and may result in severe civil and 
**  criminal penalties and will be prosecuted to the maximum extent 
**  possible under the law.
**
**  RESTRICTIONS
**
**  THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED
**  FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE
**  COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE
**  AVAILABLE TO OTHER INDIVIDUALS WITHOUT WRITTEN CONSENT
**  AND PERMISSION FROM DEVELOPER MACHINES
**
**  CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON
**  ADDITIONAL RESTRICTIONS.
**
****************************************************************************/

#ifndef QTN_GRID_GRAPHICS_H
#define QTN_GRID_GRAPHICS_H

#include <QtCore/QSet>
#include <QtCore/QBasicTimer> 
#include <QtCore/QTime>
#include <QtGui/QWidget>
#include <QtGui/qstyle.h>
#include <QtGui/qstyleoption.h>
#include <QtGui/QPainter>
#include <QtGui/QScrollBar>
#include <QtGui/QMouseEvent>
#if (QT_VERSION >= QT_VERSION_CHECK(4, 6, 0)) 
//#define Q_HAS_PROXYSTYLE
#endif

#ifdef Q_HAS_PROXYSTYLE
#include <QtGui/QProxyStyle>
#include <QtGui/QStyleFactory>
#endif

#include "QtnGridDef.h"

namespace Qtitan
{
    class GridGraphicSite;
    class GridGraphicPrimitive;
    class GridDragDropObject;
    class GridGraphicMask;
    class Effects;
#ifdef Q_HAS_PROXYSTYLE
    class GridStyle;
#endif

    enum GridDragDropState
    {
        dragNone,
        dragPreparing,
        dragActive
    };

    class QTITAN_EXPORT GridGraphicSite: public QWidget
    {
    Q_OBJECT
    public:
        friend class GridGraphicPrimitive;
        friend class GridGraphicPrimitiveScrollBar;
        friend class GridEmbedWidgetGraphic;
        GridGraphicSite(QWidget* parent);
        virtual ~GridGraphicSite();
        GridDragDropState dragDropState() const;
        GridDragDropObject* getDragDropObject() const;
    protected:
        virtual void paintEvent(QPaintEvent *);
        virtual void resizeEvent(QResizeEvent *);
        virtual void mouseMoveEvent(QMouseEvent *);
        virtual void mousePressEvent(QMouseEvent *event);
        virtual void mouseReleaseEvent(QMouseEvent* event);
        virtual void mouseDoubleClickEvent(QMouseEvent *);
        virtual void focusInEvent(QFocusEvent *);
        virtual void focusOutEvent(QFocusEvent *);
        virtual void enterEvent(QEvent *);
        virtual void leaveEvent(QEvent *);
        virtual void wheelEvent(QWheelEvent *);
        bool event(QEvent *);
        virtual void keyPressEvent(QKeyEvent *);
        virtual void keyReleaseEvent(QKeyEvent *);
        virtual void dragEnterEvent(QDragEnterEvent *);
        virtual void dragMoveEvent(QDragMoveEvent *);
        virtual void dragLeaveEvent(QDragLeaveEvent *);
        virtual void dropEvent(QDropEvent *);
        virtual void contextMenuEvent(QContextMenuEvent* event);
    protected:
        virtual void moveDragDropObject(const QPoint& pos, GridDragDropObject* object);
        virtual void stopDragDrop();
        Effects* effects() const;
    protected:
        GridGraphicPrimitive* hitTest(const QPoint& point);
        inline GridGraphicPrimitive* root() const { return m_rootGP; };
        void setRoot(GridGraphicPrimitive* root);
        void setFocusGP(GridGraphicPrimitive* gp, Qt::FocusReason reason = Qt::OtherFocusReason);
        GridGraphicPrimitive* focusedGP() const { return m_focusedGP; }
        void setActiveGP(GridGraphicPrimitive* gp);
        GridGraphicPrimitive* activeGP() const { return m_activeGP; }
        GridGraphicPrimitive* overGP() const { return m_overGP; }
        bool rectChanged(GridGraphicPrimitive* gp);
        GridGraphicPrimitive* m_overGP;
        GridGraphicPrimitive* m_pressedGP;
        GridGraphicPrimitive* m_activeGP;
        GridGraphicPrimitive* m_focusedGP;
        GridGraphicPrimitive* m_rootGP;
        GridDragDropObject* m_dragDropObject;
    private:
        void doMouseMove(const QPoint& pos, QMouseEvent* e);
        void deleteGPNotify(GridGraphicPrimitive* gp);
        GridDragDropState m_dragDropState;
        QPoint m_dragStartPos;
        Effects* m_effects;
    #ifdef Q_HAS_PROXYSTYLE
        GridStyle* m_style;
    #endif
        bool m_updateFocus;
    };


    class GridDragDropObject: public QObject
    {
        Q_OBJECT
    public:
        friend class GridGraphicSite;
        friend class GridGraphicPrimitive;
        GridDragDropObject(GridGraphicPrimitive* source);
        virtual ~GridDragDropObject();
        bool getImmediateStart();
        void setImmediateStart(bool immediateStart);
        const QCursor& cursor() const;
        void setCursor(const QCursor& cursor);
        void setDefaultCursor();
        const QCursor& acceptedCursor() const;
        void setAcceptedCursor(const QCursor& cursor);
        void setDefaultAcceptedCursor();
        virtual void beginEvent();
        virtual void moveEvent(const QPoint& pos);
        virtual void acceptEvent();
        virtual void endEvent();
        virtual void paintEvent(QPainter& painter);
        GridGraphicPrimitive* source() const;
        GridGraphicPrimitive* target() const;
        QPoint cursorPos() const;
    private:
        GridGraphicPrimitive* m_source;
        GridGraphicPrimitive* m_target;
        bool m_immediateStart;
        QCursor m_cursor;
        QCursor m_acceptedCursor;
        void setTarget(GridGraphicPrimitive* target);
    };

    struct QtnPaintData
    {
        GridGraphicPrimitive* gp;
        double opacity;
        int gx;
        int gy;
        QRect clip;
    };

    typedef QMultiMap<int, QtnPaintData> GraphicPaintOrder;

    class GridGraphicMask
    {
    public:
        GridGraphicMask(GridGraphicPrimitive* owner, const QRect& r);
        void clear() const;
        const QRect& rect() const;
        void setRect(const QRect& r);
        void addGP(GridGraphicPrimitive* gp) const;
        void removeGP(GridGraphicPrimitive* gp) const;
        bool isGPMasked(GridGraphicPrimitive* gp) const;
    protected:
    private:
        GridGraphicPrimitive* m_owner;
        QRect m_rect;
    };

    class GridGraphicPrimitive: public QObject
    {
    public:
        struct FixedState
        {
            FixedState(bool fx, bool fy) { x = fx; y = fy; }
            bool x;
            bool y;
        };
        friend class GridGraphicSite;
        friend class GridScrollableGraphicPrimitive;
        friend class GridGraphicMask;
        friend class GridDragDropObject;
        typedef QVector<GridGraphicPrimitive *> GraphicChildren;
        GridGraphicPrimitive(GridGraphicSite* site, 
            GridGraphicPrimitive* parent, const QRect& rect = QRect());
        GridGraphicPrimitive(GridGraphicPrimitive* parent, const QRect& rect = QRect());
        virtual ~GridGraphicPrimitive();
        inline GridGraphicSite* site() const { return m_site; }
        void setRect(const QRect& rect);
        void setLeft(int left);
        void setTop(int top);
        void setWidth(int width);
        void setHeight(int height);
        inline const QRect& rect() const { return m_rect; }
        inline int left() const { return m_rect.left(); }
        inline int top() const { return m_rect.top(); }
        inline int width() const { return m_rect.width(); }
        inline int height() const { return m_rect.height(); }
        void setBorder(int width);
        void setBorder(int left, int right, int top, int bottom);
        void setBorderLeft(int left);
        int borderLeft() const;
        void setBorderRight(int right);
        int borderRight() const;
        void setBorderTop(int top);
        int borderTop() const;
        void setBorderBottom(int bottom);
        int borderBottom() const;
        int leftBorderIndent() const;
        void setLeftBorderIndent(int indent);
        int rightBorderIndent() const;
        void setRightBorderIndent(int indent);
        const QColor& borderColor() const;
        void setBorderColor(const QColor & color);
        const QBrush& borderBrush() const;
        void setBorderBrush(const QBrush & brush);
        const QPen& borderPen() const;
        inline bool isMouseOver() const { return m_isMouseOver; }
        inline bool isLeftButtonPressed() const { return m_isLeftButtonPressed; }
        inline bool isLeftButtonDblClicked() const { return m_isLeftButtonDblClicked; }
        bool visibility() const;
        void setVisibility(bool visible);
        void setOpacity(double value);
        double opacity() const;
        virtual QRect viewRect() const;
        bool isDragging() const;
        QRect boundedRect() const;
        virtual QCursor cursor() const;
    #ifdef DEF_PAINT_ENGINE
        void paint(QPainter& painter, int gx, int gy, int parentScrollX, int parentScrollY, const QRect& parentRect);
    #else
        void paint(int gx, int gy, int parentScrollX, int parentScrollY, 
            double opacity, const QRect& parentRect, GraphicPaintOrder& paintOrder);
    #endif
        GridGraphicPrimitive* parentGP() const;
        void setParentGP(GridGraphicPrimitive* parent);
        const GraphicChildren& childrenGP() const;
        QPoint mapViewportToSite(const QPoint& point) const;
        QPoint mapViewportFromSite(const QPoint& point) const;
        QPoint mapViewportToGlobal(const QPoint& point) const;
        QPoint mapViewportFromGlobal(const QPoint& point) const;
        QPoint mapToSite(const QPoint& point) const;
        QPoint mapFromSite(const QPoint& point) const;
        QPoint mapToGlobal(const QPoint& point) const;
        QPoint mapFromGlobal(const QPoint& point) const;
        GridGraphicMask& mask();
        bool transparent() const;
        void setTransparent(bool transparent);
        int zIndex() const;
        void setZIndex(int zIndex);
        virtual void update(const QRect& rect);
        virtual void update();
        virtual void repaint(const QRect& rect);
        virtual void repaint();
        virtual void updateLayout();
        virtual bool scrollable() const;
        virtual bool siteHasFocus() const;
        bool childrenDrawable();
        void setChildrenDrawable(bool drawable);
        bool isUpdating() const;
    protected:
        virtual void paintEvent(QPainter& painter);
        virtual void resizeEvent(const QRect& rect);
        virtual void wheelEvent(const QPoint&, QWheelEvent *);
        /* focus events */
        virtual void focusInEvent(QFocusEvent *);
        virtual void focusOutEvent(QFocusEvent *);
        /* mouse events */
        virtual void mouseMoveEvent(const QPoint& point, QMouseEvent* e);
        virtual void mousePressEvent(const QPoint& point, QMouseEvent* e);
        virtual void mouseReleaseEvent(const QPoint& point, QMouseEvent* e);
        virtual void mouseDoubleClickEvent(const QPoint& point, QMouseEvent* e);
        virtual void mouseOverEvent(const QPoint& pos);
        virtual void mouseOutEvent();
        /* keyboard events */
        virtual void keyPressEvent(QKeyEvent* e);
        virtual void keyReleaseEvent(QKeyEvent* e);
        /* block events */
        virtual void windowBlockedEvent(QEvent* e);
        virtual void windowUnBlockedEvent(QEvent* e);
        /* drag and drop events and functions */
        virtual bool startDragDropEvent(const QPoint& pos);
        virtual void beginDragDropEvent();
        virtual bool checkDragDropEvent(const QPoint& pos, GridDragDropObject* object);
        virtual void acceptDragDropEvent(const QPoint& pos, GridDragDropObject* object);
        virtual void endDragDropEvent();
        virtual void showContextMenuEvent(const QPoint& pos, QContextMenuEvent* event);
    protected:
        void init(GridGraphicSite* site, GridGraphicPrimitive* parent, const QRect& rect);
        GridGraphicPrimitive* hitTest(const QPoint& point, int gx, int gy, int parentScrollX, int parentScrollY, const QRect& parentRect, int& zindex);
        void setCursor(const QCursor& cursor);
        QRect translateToSiteRect(const QRect& rect) const;
        QRect translateFromSiteRect(const QRect& rect) const;
        bool isLayoutChanged() const;
        void layoutChanged();
        virtual void drawBorder(QPainter& painter);
        virtual GridDragDropObject* createDragDropObject(const QPoint& point);
        GridGraphicSite* m_site;
        QRect m_rect;
        QBrush m_borderBrush;
        QPen m_borderPen;
        int m_borderLeft;
        int m_borderRight;
        int m_borderTop;
        int m_borderBottom;
        int m_leftBorderIndent;
        int m_rightBorderIndent;
    private:
        void deleteChildren();
        GridGraphicPrimitive* m_parentGP;
        GraphicChildren m_childrenGP;
        bool m_visible;
        double m_opacity;
        QCursor m_cursor;
        bool m_isMouseOver;
        bool m_isLeftButtonPressed;
        bool m_isLeftButtonDblClicked;
        GridGraphicMask m_graphicMask;
        bool m_transparent;
        int m_zIndex;
        bool m_layoutChanged;
        FixedState m_fixedState;
        bool m_masked;
        bool m_childrenDrawable;
        bool m_isUpdating;
    public:
        static void drawBorderPrimitive(QPainter& painter,
                                        QPen pen, 
                                        const QRect& rect,
                                        const QRect& clip,
                                        int borderLeft, int borderRight,
                                        int borderTop, int borderBottom);
    };

    enum ScrollOverflow
    {
        gsoNone,
        gsoVisible,
        gsoAuto
    };

    class GridScrollableGraphicPrimitive: public GridGraphicPrimitive
    {
        Q_OBJECT
    public:
        friend class GridGraphicSite;
        GridScrollableGraphicPrimitive(GridGraphicSite* site, 
            GridGraphicPrimitive* parent, const QRect& rect = QRect());
        virtual ~GridScrollableGraphicPrimitive();
        QScrollBar* scrollVer() { return m_scrollVer; };
        QScrollBar* scrollHor() { return m_scrollHor; };
        int scrollX() const { return m_scrollX; }
        int scrollY() const { return m_scrollY; }
        void setScrollX(int x);
        void setScrollY(int y);
        inline ScrollOverflow overflowX() const { return m_overflowX; }
        inline ScrollOverflow overflowY() const { return m_overflowY; }
        void setOverflow(ScrollOverflow overflow);
        void setOverflowX(ScrollOverflow overflow);
        void setOverflowY(ScrollOverflow overflow);
        const QSize& viewSize() const;
        void setViewSize(const QSize& size);
        virtual QRect viewRect() const;
        QRect boundedRect() const;
        void setFixedGPX(GridGraphicPrimitive* primitive, bool value);
        void setFixedGPY(GridGraphicPrimitive* primitive, bool value);
        bool isGPFixedX(const GridGraphicPrimitive* primitive) const;
        bool isGPFixedY(const GridGraphicPrimitive* primitive) const;
        virtual void updateLayout();
        virtual bool scrollable() const;
        virtual bool canScrollX() const;
        virtual bool canScrollY() const;
        virtual void updateScrollBars();
        QSize minBoundedSize() const;
    protected:
        void setScrollBarPos();
    protected:
        virtual void wheelEvent(const QPoint&, QWheelEvent *);
        virtual void resizeEvent(const QRect& rect);
        virtual void scrollXEvent(int position);
        virtual void scrollYEvent(int position);
    private:
        QScrollBar* m_scrollVer;
        QScrollBar* m_scrollHor;
        int m_scrollX;
        int m_scrollY;
        ScrollOverflow m_overflowX;
        ScrollOverflow m_overflowY;
        QSize m_viewSize;
    public slots:
        void scrollVerChanged(int value);
        void scrollHorChanged(int value);
    };

    class PaintBuffer
    {
    public:
        PaintBuffer();
        void setSize(const QSize& size);
        QPainter& begin();
        void end();
        const QImage& buffer() const;
        bool empty() const;
        void reset();
    private:
        QImage m_buffer;
        QPainter m_painter;
        bool m_empty;
    };

    class GridBufferedGraphicPrimitive: public GridGraphicPrimitive
    {
    public:
        GridBufferedGraphicPrimitive(GridGraphicSite* site, 
            GridGraphicPrimitive* parent, const QRect& rect = QRect());
        virtual void update(const QRect& rect);
    protected:
        virtual void paintEvent(QPainter& painter);
        virtual void resizeEvent(const QRect& rect);
        virtual void paintBufferEvent(QPainter& painter) = 0;
    protected:
        void resetPaintBuffer();
    private:
        PaintBuffer m_paintBuffer;
    };

    class GridEmbedWidgetGraphic: public GridGraphicPrimitive
    {
    public:
        friend class GridGraphicSite;
        GridEmbedWidgetGraphic(GridGraphicSite* site, 
            GridGraphicPrimitive* parent, const QRect& rect = QRect());
        virtual ~GridEmbedWidgetGraphic();
        void setWidget(QWidget* widget);
        QWidget* widget() const;
        virtual bool siteHasFocus() const;
    protected:
        virtual void paintEvent(QPainter& painter);
        virtual void resizeEvent(const QRect& rect);
        void updateWidgetGeometry();
        virtual bool eventFilter(QObject *, QEvent *);
        /* mouse events */
        virtual void mouseMoveEvent(const QPoint& point, QMouseEvent* e);
        virtual void mousePressEvent(const QPoint& point, QMouseEvent* e);
        virtual void mouseReleaseEvent(const QPoint& point, QMouseEvent* e);
        virtual void mouseDoubleClickEvent(const QPoint& point, QMouseEvent* e);
        virtual void mouseOverEvent(const QPoint& pos);
        virtual void mouseOutEvent();
        virtual void wheelEvent(const QPoint&, QWheelEvent *);
        /* keyboard events */
        virtual void keyPressEvent(QKeyEvent* e);
        virtual void keyReleaseEvent(QKeyEvent* e);
        /* focus events */
        virtual void focusInEvent(QFocusEvent *);
        virtual void focusOutEvent(QFocusEvent *);
        virtual void showContextMenuEvent(const QPoint& pos, QContextMenuEvent* event);
        virtual QCursor cursor() const;
    private:
        void installFilterToWidget(QWidget* widget);
        void removeFilterFromWidget(QWidget* widget);
        bool sendFocusInEvent(QWidget* w, Qt::FocusReason reason);
        bool sendFocusOutEvent(QWidget* w, Qt::FocusReason reason);
        void doMouseOverEvent(const QPoint& pos);
        void doMouseOutEvent();
        void connectWidget();
        void disconnectWidget();
        void setInternalWinPos() const;
        QWidget* getOverWidget(const QPoint& pos, QPoint& widgetPos);
        void setFocusChild(QWidget* fw);
        QWidget* m_widget;
        QSet<QWidget*> m_filteredWidgets;
        QWidget* m_overWidget;
        QWidget* m_pressedWidget;
        QWidget* m_focusedWidget;
        QImage m_paintBuffer;
    private:
        static QHash<QWidget *, GridEmbedWidgetGraphic *> allEmbedWidgets;
        static bool qapplication_callback(void** context);
    };

    class GridGraphicPrimitiveScrollBar: public QScrollBar
    {
        Q_OBJECT
    public:
        friend class GridScrollableGraphicPrimitive;
        GridGraphicPrimitiveScrollBar(Qt::Orientation, GridScrollableGraphicPrimitive* parent);
    protected:
        virtual void mouseMoveEvent(QMouseEvent *);
        virtual void mousePressEvent(QMouseEvent *);
    private:
        GridScrollableGraphicPrimitive* m_scrollableGraphic;
    };

    extern int mul_div(int a, int b, int c);
    extern void draw_text(QPainter& painter, const QString& text, const QRect& rect, int flags, bool transparent = true);
    extern bool intersect_line(QLineF& lineF, const QRect& region);
    extern void draw_line(QPainter& painter, const QLineF& line);
    extern void draw_select_frame(QPainter& painter, const QColor& color, const QRect& rect);
    extern void set_clip_reg(QPainter& painter, const QRect& rect);
    extern QColor qcolor_mix_rgb(const QColor& first, const QColor& second);
    extern QColor qcolor_mix_cmy(const QColor& first, const QColor& second);
    extern QVariant::Type QTITAN_EXPORT qvariant_to_pixmap(const QVariant& v, QPixmap& pixmap);

    #ifdef Q_HAS_PROXYSTYLE
    class GridStyle: public QProxyStyle
    {
    public:
        GridStyle(QStyle* style);
        virtual void drawPrimitive(PrimitiveElement pe, const QStyleOption *opt, QPainter *p,
                                   const QWidget *w = 0) const;
        virtual void drawControl(ControlElement element, const QStyleOption *opt, QPainter *p,
                                 const QWidget *w = 0) const;
    private:
        typedef QHash<QStyleOptionButton, QImage> ButtonControlFragmentsHash;
        typedef QHash<QStyleOptionButton, QImage> ButtonFragmentsHash;
        ButtonControlFragmentsHash m_buttonControlFragmets;
        ButtonFragmentsHash m_buttonFragmets;
        static ButtonControlFragmentsHash::const_iterator addToControlButtonHash(GridStyle* style, 
            const QStyleOptionButton& option, const QImage& image);
        static ButtonFragmentsHash::const_iterator addToButtonHash(GridStyle* style, 
            const QStyleOptionButton& option, const QImage& image);
        bool canCachedPrimitive(const QWidget* w, PrimitiveElement pe, const QStyleOption *opt) const;
        bool canCachedControl(const QWidget* w, ControlElement element, const QStyleOption *opt) const;
    };
    #endif

    class SmoothEffect
    {
    public:
        friend class Effects;
        SmoothEffect();
        void intialize(GridGraphicPrimitive* gp);
        void setInitialState(const QImage& iState);
        int duration() const;
        void setDuration(int duration);
        bool isActive() const;
        void paint(QPainter* painter, const QImage& currentState);
        void changeStateToCurrent();
    private:
        QImage m_istate;
        QImage m_fg;
        QImage m_mixed;
        GridGraphicPrimitive* m_gp;
        int m_duration;
        bool m_active;
        QTime m_time;
    };

    class Effects: public QObject
    {
    public:
        Effects(QWidget* site);
        virtual ~Effects();
        void run(SmoothEffect* effect);
        void stop(SmoothEffect* effect);
        void update();
        SmoothEffect* findEffect(GridGraphicPrimitive* gp);
    protected:
        virtual bool event(QEvent* e);
    private:
        QWidget* m_site;
        QList<SmoothEffect *> m_list;
        QBasicTimer m_timer;
    };

    class EffectSettings
    {
    public:
        EffectSettings();
        virtual ~EffectSettings();
        void setStyleOption(QStyleOption* option);
        QStyleOption* styleOption() const;
        QSize size() const;
        void setBg(const QImage& bg);
        const QImage& bg() const;
    private:
        QStyleOption* m_opt;
        QImage m_bg;
    };

#ifdef Q_OS_MAC
    void set_widget_mac_hook(GridEmbedWidgetGraphic* gp, QWidget* w);
    void unset_widget_mac_hook(GridEmbedWidgetGraphic* gp, QWidget* w);
#endif

}; //namespace Qtitan

#endif //QTN_GRID_GRAPHICS_H


