/****************************************************************************
**
** Qtitan Library by Developer Machines (Advanced Grid for Qt)
** 
** Copyright (c) 2009-2011 Developer Machines (http://www.devmachines.com)
**           ALL RIGHTS RESERVED
** 
**  The entire contents of this file is protected by copyright law and
**  international treaties. Unauthorized reproduction, reverse-engineering
**  and distribution of all or any portion of the code contained in this
**  file is strictly prohibited and may result in severe civil and 
**  criminal penalties and will be prosecuted to the maximum extent 
**  possible under the law.
**
**  RESTRICTIONS
**
**  THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED
**  FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE
**  COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE
**  AVAILABLE TO OTHER INDIVIDUALS WITHOUT WRITTEN CONSENT
**  AND PERMISSION FROM DEVELOPER MACHINES
**
**  CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON
**  ADDITIONAL RESTRICTIONS.
**
****************************************************************************/

#ifndef QTN_GRIDTABLEVIEW_H
#define QTN_GRIDTABLEVIEW_H

#include <QTimer>
#include <QMenu>

#include "QtnGrid.h"
#include "QtnGridEditors.h"

namespace Qtitan
{
    class GridTableView;
    class GridTableColumnBase;
    class GridTableColumn;
    class GridTableDragColumnObject;
    class GridTableViewFixedDecorGraphic;
    class QuickCustomizationPopup;
    class GridTableViewOptions;
    class GridVirtualTableEngine;
    class GridTableVirtualRow;
    class GridTableQuickRowButtonGraphic;
    class GridTableDataRowGraphic;
    class GridTableGroupRowGraphic;
    class GridTableGroupIndentGraphic;
    class GridTableCellGraphic;
    class GridTableGroupSummaryGraphic;
    class GridTableSummaryCellGraphic;
    class GridNewRowGraphic;
    class GridNewRowDelimeterGraphic;
    class GridContextMenu;
    class GridSummaryContextMenu;
    class GridTableColumnEditor;
    class GridColumnEditorGraphic;
    class GridTableColumnGraphic;
    class GridTableGroupPanelGraphic;

    enum ColumnFixedKind
    {
        FixedNone,
        FixedLeft,
        FixedRight
    };

    class GridTableViewGraphic: public GridViewGraphic
    {
        Q_OBJECT
    public:
        GridTableViewGraphic(GridTableView* view);
        virtual void requestUpdateLayout(bool recalculateHeight = false);
        virtual void updateLayout();
        void getHeaderColumnListFromRange(int x1, int x2, QList<GridTableColumn *>& columns);
        bool hasFixedLeft();
        GridTableView* view() const;
    protected:
        virtual void resizeEvent(const QRect& rect);
        virtual void scrollXEvent(int position);
        virtual void scrollYEvent(int position);
        virtual QSize updateNewRowLayout();
        virtual QSize updateFooterLayout();
        virtual void updateScrollBars();
    protected:
        virtual void keyPressEvent(QKeyEvent* e);
        virtual void showContextMenuEvent(const QPoint& pos, QContextMenuEvent* event);
    protected:
        virtual bool canScrollY() const;
    private slots:
        void updateYScrollByTimeout();
    private:
        QTimer* m_updateYScrollTimeout;
    };

    enum GridTableActionType
    {
        GridExpandAll = 0,
        GridCollapseAll,
        GridSortNone,
        GridSortAscending,
        GridSortDescending,
        GridHideColumn,
        GridGroupByColumn,
        GridGroupByBox,
        GridFieldChooser,
        GridFooter,
        GridGroupFooters,
        GridAlignmentGroup,
        GridAlignLeft,
        GridAlignRight,
        GridAlignCenter
    };

    class QTITAN_EXPORT GridTableView: public GridViewBase
    {
        Q_OBJECT
    public:
        friend class Grid;
        friend class GridTableViewGraphic;
        friend class GridTableColumnBase;
        friend class GridTableColumn;
        friend class GridTableColumnGraphic;
        friend class GridTableGroupPanelGraphic;
        friend class GridTableHeaderGraphic;
        friend class GridTableDragColumnObject;
        friend class GridTableColumnSizingObject;
        friend class GridTableSelectionObject;
        friend class GridTableViewOptions;
        friend class GridTableViewFixedDecorGraphic;
        friend class QuickCustomizationPopup;
        friend class GridTableVirtualRow;
        friend class GridTableDataRowGraphic;
        friend class GridTableSummaryCellGraphic;
        friend class GridTableSummaryGraphicBase;
        friend class GridTableGroupSummaryCellGraphic;
        friend class GridTableFooterSummaryCellGraphic;
        friend class GridTableFooterGraphic;
        friend class GridTableCellGraphic;
        friend class GridVirtualTableEngine;
        friend class GridNewRowGraphic;
        friend class GridNewRowDelimeterGraphic;
        friend class GridTableQuickRowButtonGraphic;
        friend class GridTableGroupRowGraphic;
        GridTableColumn* addColumn(const QString& modelColumnName, const QString& caption = QString());
        GridTableColumn* addColumn(int modelColumn, const QString& caption = QString());
        static GridTableView* create(Grid* grid);
        virtual bool isColumnAvailable(const GridColumnBase* column) const; 
        typedef QList<GridTableColumn *> GridTableColumns;
        typedef QSet<GridTableColumn *> GridColumnSet;
        virtual void columnsChanged();
        virtual void textAlignmentChanged(GridColumnBase* columnBase);
        int groupColumnCount() const;
        GridTableColumn* groupColumnByIndex(int groupIndex) const;
        GridTableColumns& groups() { return m_groups; }
        void clearGrouping();
        GridTableViewOptions& options() const;
        virtual void beginUpdate();
        virtual void endUpdate();
        GridContextMenu* contextMenu() const;
#ifndef QTN_NOUSE_XML_MODULE
    protected:
        bool saveLayoutToXML(QXmlStreamWriter* xmlwriter);
        bool loadLayoutFromXML(QXmlStreamReader* xmlreader);
        virtual bool saveLayoutToStream(QXmlStreamWriter* xmlwriter);
        virtual bool loadLayoutFromStream(QXmlStreamReader* xmlreader);
        virtual bool saveTableColumnLayout(const GridTableColumn* column, QXmlStreamWriter* xmlwriter);
        virtual bool loadTableColumnLayout(GridTableColumn* column, QXmlStreamReader* xmlreader);
        bool loadColumnBaseLayout(GridColumnBase* column, QXmlStreamReader* xmlreader);
        bool saveColumnBaseLayout(const GridColumnBase* column, QXmlStreamWriter* xmlwriter);
        bool isLayoutLoading() const;
#endif
    public:
        virtual void navigatePageUp(Qt::KeyboardModifiers modifiers  = Qt::NoModifier);
        virtual void navigatePageDown(Qt::KeyboardModifiers modifiers = Qt::NoModifier);
        virtual void navigateUp(Qt::KeyboardModifiers modifiers = Qt::NoModifier);
        virtual void navigateDown(Qt::KeyboardModifiers modifiers = Qt::NoModifier);
        virtual void navigateLeft(Qt::KeyboardModifiers modifiers = Qt::NoModifier);
        virtual void navigateRight(Qt::KeyboardModifiers modifiers = Qt::NoModifier);
        virtual void navigateTab(Qt::KeyboardModifiers modifiers = Qt::NoModifier);
        virtual GridTableColumn* getFirstColumn();
        virtual GridTableColumn* getLastColumn();
        virtual GridTableColumn* getNextColumn(const GridTableColumn* anchor);
        virtual GridTableColumn* getPrevColumn(const GridTableColumn* anchor);
        bool showEditor();
        void hideEditor();
        bool closeEditor();
        bool isEditing() const;
        bool isModified() const;
        bool showNewRowEditor();
        bool closeNewRowEditor();
        void hideNewRowEditor();
        bool isNewRowEditing() const;
        GridRow* getRow(const QModelIndex& index) const;
        GridRow* getRow(int rowIndex) const;
        int getRowCount() const;
        GridRow* focusedRow();
        int focusedRowIndex();
        bool setFocusedRowIndex(int rowIndex, Qt::KeyboardModifiers modifiers = Qt::NoModifier, GridEditorFocusPolicyFlag flag = NoPolicy);
        int focusedColumnIndex();
        bool setFocusedColumnIndex(int columnIndex, Qt::KeyboardModifiers modifiers = Qt::NoModifier, GridEditorFocusPolicyFlag flag = NoPolicy);
        void selectRow(int rowIndex, Qt::KeyboardModifiers modifiers = Qt::NoModifier);
        void selectRowRange(int firstRowIndex, int lastRowIndex, Qt::KeyboardModifiers modifiers = Qt::NoModifier);
        void selectRange(int firstRowIndex, int lastRowIndex,
                         int firstColumnIndex, int lastColumnIndex,
                         Qt::KeyboardModifiers modifiers = Qt::NoModifier);
        void selectRange(GridRow* firstRow,
                         GridRow* lastRow, 
                         const QList<GridTableColumn *>& columns,
                         Qt::KeyboardModifiers modifiers = Qt::NoModifier);
        void selectAll();
        GridSelection* selection() const;
        GridSelectionIterator* selectionIterator() const;
    public slots:
        void expandAll();
        void collapseAll();
    Q_SIGNALS:
        void focusRowChanged(QObject* sender, int oldRowIndex, int rowIndex);
        void focusColumnChanged(QObject* sender, int oldColumnIndex, int columnIndex);
        void summaryCalculated(QObject* sender);
        void summaryFooterTextChanging(QObject* sender, SummaryTextEventArgs* args);
        void summaryGroupTextChanging(QObject* sender, SummaryGroupTextEventArgs* args);
    public:
        GridTableView(Grid* grid);
        virtual ~GridTableView();
    protected:
        virtual void init();
        virtual int columnGroupIndex(const GridColumnBase* column) const;
        virtual void setColumnGroupIndex(GridColumnBase* column, int groupIndex, bool caseSensitivity, bool localeAware);
        void setColumnWidth(GridTableColumnBase* column, int newWidth);
        GridEditorRepository* createEditorRepository(const GridTableColumn* column);
        bool startEditCell(int rowIndex, int columnIndex, bool& editContextChanged);
        void cancelEditCell();
        bool closeEditorWithoutUpdate();
        virtual bool hasVisibleParent(const GridColumnBase* column) const; 
        virtual int getVisualIndex(const GridTableColumnBase* columnBase) const;
        virtual void setVisualIndex(GridTableColumnBase* tableColumn, int visualIndex);
        virtual int getColumnRowSpan(const GridTableColumn* column) const;
        virtual void setColumnRowSpan(GridTableColumn* column, int rowSpan);
        virtual int getColumnRowIndex(const GridTableColumn* column) const;
        GridTableColumnBase* getPrevious(const GridTableColumnBase* tableColumn) const;
        GridTableColumnBase* getNext(const GridTableColumnBase* tableColumn) const;
        virtual GridTableColumnBase* getPreviousResizeColumn(const GridTableColumnBase* column) const;
        virtual GridTableColumnBase* getNextResizeColumn(const GridTableColumnBase* column) const;
        virtual bool isVerticalResizable(const GridTableColumnBase* columnBase) const;
        virtual bool isHorizontalResizable(const GridTableColumnBase* columnBase) const;
        virtual ColumnFixedKind getColumnFixedKind(const GridTableColumnBase* columnBase) const;
        virtual void setColumnFixedKind(GridTableColumnBase* columnBase, ColumnFixedKind kind);
        virtual GridColumnBase* createColumn(int modelColumn, const QString& caption);
        virtual void getColumnLine(const GridColumnBase* column, GridColumns& retval) const;
        virtual int getMaximumVisualIndex(const GridTableColumnBase* anchor) const;
        virtual void columnWidthChanged(GridColumnBase* column);
        virtual int getColumnViewWidth(GridTableColumnBase* column);
        virtual int columnMinWidth(GridTableColumnBase* column);
        virtual int columnMaxWidth(GridTableColumnBase* column);
        virtual bool getColumnVisible(const GridColumnBase* columnBase) const;
        virtual void setColumnVisible(GridColumnBase* columnBase, bool visible);
        virtual GridViewOptions* createOptions();
        virtual GridHeaderPanelGraphic* createHeaderPanelGraphic();
        virtual GridFooterPanelGraphic* createFooterPanelGraphic();
        virtual GridRowsGraphic* createRowsGraphic();
        virtual void columnCreated(const GridColumnBase* column);
        virtual void updateViewFixedMask();
        void updateViewFixedMaskEx(int leftIndent, int rightIndent);
        virtual int quickButtonIndent() const;
        virtual void getFixedColumns(GridTableColumnBase*& left, GridTableColumnBase*& right) const;
        virtual bool checkRemove(const GridTableColumnBase* columnBase) const;
        virtual bool checkMove(const GridTableColumnBase* columnBase) const;
        virtual bool checkHorSizing(const GridTableColumnBase* columnBase) const;
        virtual bool checkGrouping(const GridTableColumnBase* columnBase) const;
        virtual void controllerChanged();
        virtual int getColumnHeight(GridColumnBase* columnBase) const;
        int actualWidth() const;
        bool fullyShowingOnView(GridTableColumnBase* column) const;
        void resetCalculatedModelHeight();
        void calculateModelHeight();
        int getModelHeight();
        int getRowLeftIndent(GridRow* gridRow) const;
        int getColumnHeaderTop() const;
        int getColumnHeaderWidth() const;
        int getColumnHeaderHeight() const;
        const QRect& getFooterPanelRect() const;
        void updateRows(bool forceUpdate = false);
        QRect calculateRowsRect(const QSize& viewSize) const;
        void createContextMenu();
        GridSummaryContextMenu* createSummaryContextMenu(QMap< SummaryType, QAction*>& summaryActions) const;
        bool setFocusedCell(int rowIndex, int columnIndex, Qt::KeyboardModifiers modifiers = Qt::NoModifier, GridEditorFocusPolicyFlag flag = NoPolicy);
    protected:
        virtual GridViewGraphic* createGraphic();
        virtual bool hasGroupingIndent(const GridTableColumnBase* column) const;
        virtual int getColumnLineCount() const;
        bool getAutoHeightRowCount(int& rowCount) const;
        void calculateAutoHeight(GridRow* gridRow, int* heights) const;
        void updateAutoHeight();
        void editorChanging(const GridTableColumn* column);
        void scrollToRow(GridRow* row, bool topPosition);
    protected:
        GridVirtualTableEngine* m_tableEngine;
    private:
        bool checkEditorPolicy(GridEditorFocusPolicy policy);
        bool isFocusedTableCell(int rowIndex, int columnIndex) const;
        int getNextPageRowIndex(int rowIndex, bool up, int* offsetY = Q_NULL);
        int getSummaryHeight() const;
        int getDataRowHeight() const;
        int getGroupRowHeight() const;
        void raiseFocusedRowChanged(int oldRow, int newRow);
        void raiseSummaryFooterTextChanging(SummaryTextEventArgs* args);
        void raiseSummaryGroupTextChanging(SummaryGroupTextEventArgs* args);
        void summaryNone(GridTableSummaryCellGraphic* cell);
        void summarySum(GridTableSummaryCellGraphic* cell);
        void summaryMin(GridTableSummaryCellGraphic* cell);
        void summaryMax(GridTableSummaryCellGraphic* cell);
        void summaryCount(GridTableSummaryCellGraphic* cell);
        void summaryAvg(GridTableSummaryCellGraphic* cell);
        GridTableColumns m_groups;
        GridFieldChooserProvider* m_columnProvider;
        GridTableViewFixedDecorGraphic* m_decorLayer;
        int m_modelHeight;
        GridContextMenu* m_contextMenu;
        QMenu* m_alignmentMenu;
        QMap<GridTableActionType, QAction*> m_actions;
        struct EditContext
        {
            int modelRowIndex;
            int columnIndex;
            QVariant value;
            bool modified;
            GridTableCellGraphic* editingCell;
            EditContext() : modelRowIndex(-1), columnIndex(-1), modified(false), editingCell(Q_NULL) {}
        };
        EditContext m_editingContext;

        bool m_isNewRowEditing;
        bool m_isNewRowModify;
        int m_shiftColumnIndex;
        int m_shiftRowIndex;
#ifndef QTN_NOUSE_XML_MODULE
        bool m_layoutLoading;
#endif
    protected:
        int m_updateFocusStack;
    private slots:
        void sortNone();
        void sortAscending();
        void sortDescending();
        void hideColumn();
        void groupByColumn();
        void groupByBox();
        void fieldChooser();
        void showHideFooter();
        void showHideGroupFooters();
        void alignLeftAct();
        void alignRightAct();
        void alignCenterAct();
    };

    class QTITAN_EXPORT GridTableColumnBase: public GridColumnBase
    {
    public:
        friend class GridTableView;
        friend class GridTableColumnSizingObject;
        friend class GridTableDragColumnObject;
        friend class GridTableSelectionObject;
        friend class GridTableViewGraphic;
        friend class GridTableColumnGraphic;
        friend class GridTableHeaderGraphic;
        friend class GridColumnElementGraphic;
        friend class GridTableViewFixedDecorGraphic;
        friend class GridTableDataRowGraphic;
        inline GridTableView* view() const { return GridColumnBase::view<GridTableView>(); }
        int minWidth();
        int maxWidth();
        virtual int visualIndex() const;
        virtual void setVisualIndex(int visualIndex);
        virtual bool isShowingFirst() const;
        virtual bool isShowingLast() const;
        ColumnFixedKind fixedKind() const;
        void setFixedKind(ColumnFixedKind kind);
        bool isHidingEnabled() const;
        void setHidingEnabled(bool enabled);
        bool isMovingEnabled() const;
        void setMovingEnabled(bool enabled);
        bool isHorSizingEnabled() const;
        void setHorSizingEnabled(bool enabled);
        int width() const;
        void setWidth(int width);
        GridTableColumnEditor* columnEditor() const;
        GridTableColumnEditor* setColumnEditor(GridTableColumnEditor* editor);
    protected:
        GridTableColumnBase(GridTableView* view, const QString& caption);
        virtual ~GridTableColumnBase();    
        virtual GridTableDragColumnObject* createDragDropColumnObject(const QPoint& point, GridGraphicPrimitive* graphic) = 0;
        virtual bool needHeaderHints() const = 0;
        virtual void initStyleOption(QStyleOptionHeader& styleOption);
        GridTableColumnBase* getPreviousResizeColumn() const;
        GridTableColumnBase* getNextResizeColumn() const;
        void changeWidth(int width);
        LayoutZone getLayoutZone(const QPoint& point, bool testUpDown = true) const;
        virtual void ensureWidth();
        ColumnFixedKind m_fixedKind;
        int m_graphicWidth;
    private:
        bool m_isHidingEnabled;
        bool m_isMovingEnabled;
        bool m_isHorSizingEnabled;
        GridTableColumnEditor* m_columnEditor;
    };

    class GridColumnEditorGraphic: public GridEmbedWidgetGraphic
    {
    public:
        friend class GridTableColumnGraphic;
        GridColumnEditorGraphic(GridTableColumnGraphic* parent, GridTableColumnBase* column);
        virtual ~GridColumnEditorGraphic();
    protected:
        virtual void mousePressEvent(const QPoint& point, QMouseEvent* e);
        virtual void mouseOverEvent(const QPoint& pos);
        virtual void mouseOutEvent();
    private:
        GridTableColumnBase* m_column;
    };

    class QTITAN_EXPORT GridTableColumn: public GridTableColumnBase
    {
        Q_OBJECT
    public:
        friend class GridTableDragColumnObject;
        friend class GridTableView;
        friend class GridTableVirtualRow;
        friend class GridTableSummaryGraphicBase;
        friend class GridVirtualTableEngine;
        friend class GridTableCellGraphic;
        GridTableColumn(GridTableView* view, const QString& caption);
        virtual ~GridTableColumn();
        SortOrder sortOrder() const;
        void setSortOrder(SortOrder order);
        bool isSortLocaleAware() const;
        void setSortLocaleAware(bool value);
        bool sortCaseSensitivity() const;
        void setSortCaseSensitivity(bool value);
        bool isSortEnabled() const;
        void setSortEnabled(bool enabled);
        bool isGroupSortEnabled() const;
        void setGroupSortEnabled(bool enabled);
        bool isGroupingEnabled() const;
        void setGroupingEnabled(bool enabled);
        bool isHidingOnGroupingEnabled() const;
        void setHidingOnGroupingEnabled(bool enabled);
        int groupIndex() const;
        void setGroupIndex(int groupIndex);
        GridEditorRepository* editorRepository();
        void setEditorRepository(GridEditorRepository* repository);
        GridEditorType editorType() const;
        void setEditorType(GridEditorType type);
        void setFooterSummary(SummaryType type);
        void setGroupSummary(GridTableColumn* groupColumn, SummaryType type);
        void setDefaultGroupSummary(SummaryType type);
    protected:
        virtual void init();
        int sortIndex() const;
        void setSortIndex(int sortIndex);
        virtual void click(QMouseEvent* e);
        virtual GridTableDragColumnObject* createDragDropColumnObject(const QPoint& point, GridGraphicPrimitive* graphic);
        virtual bool needHeaderHints() const;
        void setLayouterRow(int row);
        int layouterRow() const;
    private:
        bool m_isSortLocaleAware;
        bool m_sortCaseSensitivity;
        bool m_isSortEnabled;
        bool m_isGroupSortEnabled;
        bool m_isGroupingEnabled;
        bool m_isHidingOnGroupingEnabled;
        int m_layouterRow;
        GridEditorRepository* m_editorRepository;
        GridEditorType m_editorType;
    };

    class GridStretchWidthLayout
    {
    public:
        struct StretchWidthInfo
        {
            int minWidth;
            int width;
            int newWidth;
            bool ignore;
            StretchWidthInfo(int min, int w, bool ign)
            : minWidth(min), width(w), newWidth(-1), ignore(ign) { }
        };
        StretchWidthInfo& getInfoByIndex(int i);
        int infoCount();
        int widthSum();
        void addColumnInfo(int min, int width, bool ignore);
        void execute();
        GridStretchWidthLayout();
        void setViewWidth(int viewWidth);
    private:
        QList<StretchWidthInfo> m_infolist;
        int m_viewWidth;
    };

    class GridTableColumnSizingObject: public GridDragDropObject
    {
    public:
        GridTableColumnSizingObject(GridTableColumnBase* column, Qt::Orientation orient);
    protected:
        virtual void beginEvent();
        virtual void moveEvent(const QPoint& pos);
        virtual void endEvent();
        virtual void paintEvent(QPainter& painter);
    private:
        QRect calculatePaintRect();
        QRect m_paintRect;
        QPoint m_rightTop;
        QPoint m_screenPos;
        GridTableColumnBase* m_column;
        int m_deltaX;
        int m_deltaY;
        Qt::Orientation m_orient;
    };
    
    class GridTableDragColumnObject: public GridDragDropObject
    {
        Q_OBJECT
    public:
        GridTableDragColumnObject(GridTableView* view, const QPoint& point, GridTableColumnBase* column, GridGraphicPrimitive* graphic);
        virtual ~GridTableDragColumnObject();
        void setSourceGroupIndex(int groupIndex);
        int sourceGroupIndex() const;
        void setSourceVisualIndex(int visualIndex);
        int sourceVisualIndex() const;
        bool hideColumnFlag() const;
        void setHideColumnFlag(bool flag);
        virtual bool checkDropColumnTarget(const QPoint& pos, GridTableColumnBase* target);
        virtual bool checkDropHeaderTarget(const QPoint& pos, GridTableGroupPanelGraphic* target);
        virtual void clear();
        DragColumnWidget* dragColumnWidget() const;
    protected:
        virtual void beginEvent();
        virtual void moveEvent(const QPoint& pos);
        virtual void acceptEvent();
        virtual void endEvent();
        virtual void calculateViewCoordinates(GridTableColumnBase* target, QPoint& viewPos, int& viewWidth) const;
        bool updateFieldChooserWindow(const QPoint& gPos);
    protected:
        GridTableView* m_view;
        GridTableColumnBase* m_column;
    private:
        QPoint m_startPos;
        DragColumnWidget* m_dragColumnWidget;
        QTimer* m_scrollTimer;
        QCursor m_removeCursor;
        QCursor m_scrLeftCursor;
        QCursor m_scrRightCursor;
        int m_sourceGroupIndex;
        int m_sourceVisualIndex;
        bool m_hideColumnFlag;
    protected:
        void showUpDownArrows(GridTableColumnBase* target, LayoutZone zone);
        ColumnArrowWidget* m_upArrow;
        ColumnArrowWidget* m_downArrow;
        ColumnArrowWidget* m_leftArrow;
        ColumnArrowWidget* m_rightArrow;
    private slots:
        void scrollToCursor();
    };

    class GridTableSelectionRubberBand: public GridGraphicPrimitive
    {
    public:
        GridTableSelectionRubberBand(GridGraphicPrimitive* parent);
    protected:
        virtual void paintEvent(QPainter& painter);
    };

    class GridTableSelectionObject: public GridDragDropObject
    {
    public:
        GridTableSelectionObject(GridTableView* view, const GridCell& cell, const QPoint& cellPos);
        virtual ~GridTableSelectionObject();
        void selectTo(const QPoint& pos);
        void scrollTo(const QPoint& pos);
        void setRubberBandRect(const QPoint& pos);
    protected:
        virtual bool event(QEvent* e);
        virtual void beginEvent();
        virtual void moveEvent(const QPoint& pos);
        virtual void endEvent();
    private:
        GridTableView* m_view;
        GridTableSelectionRubberBand* m_rubberBandGP;
        GridCell m_startCell;
        GridCell m_endCell;
        QPoint m_cellPos;
        QPoint m_lastPos;
        QBasicTimer m_scrollTimer;
    };

    class GridTableViewFixedDecorGraphic: public GridGraphicPrimitive
    {
    public:
        GridTableViewFixedDecorGraphic(GridTableView* view);
    protected:
        virtual void paintEvent(QPainter& painter);
    private:
        GridTableView* m_view;
    };

    enum GridLinesStyle
    {
        LinesNone = 0,
        LinesBoth,
        LinesHorizontal,
        LinesVertical
    };

    enum GridNewRowPlace
    {
        NewRowNone = 0,
        NewRowTop,
        NewRowBottom
    };

    enum GridSelectionPolicy
    {
        SelectSingleRow = 0,
        SelectSingleCell,
        SelectMultiRow,
        SelectMultiCell
    };

    enum RowHighlightEffect
    {
        RowFlashEffect,
        RowAlphaEffect
    };

    enum GridGroupRowStyle
    {
        GridGroupStandard,
        GridGroupOffice11
    };

    class QTITAN_EXPORT GridTableViewOptions: public GridViewOptions
    {
        Q_OBJECT
    protected:
        friend class GridTableView;
        GridTableViewOptions(GridTableView* view);
    public:
        bool columnAutoWidth() const;
        void setColumnAutoWidth(bool autoWidth);
        bool columnsHeader() const;
        void setColumnsHeader(bool columnsHeader);
        bool columnsQuickCustomization() const;
        void setColumnsQuickCustomization(bool quickCustomization);
        int columnsQuickCustomizationMaxDropDownCount() const;
        void setColumnsQuickCustomizationMaxDropDownCount(int dropDownCount);
        bool columnHeaderHints() const;
        void setcolumnHeaderHints(bool headerHints);
        const QColor& fixedSeparatorColor() const;
        void fixedSeparatorColor(const QColor& color);
        int fixedSeparatorWidth() const;
        void fixedSeparatorWidth(int width);
        bool isColumnHidingEnabled() const;
        void setColumnHidingEnabled(bool enabled);
        bool isColumnMovingEnabled() const;
        void setColumnMovingEnabled(bool enabled);
        bool isColumnHidingOnGroupingEnabled() const;
        void setColumnHidingOnGroupingEnabled(bool enabled);
        bool isColumnHorSizingEnabled() const;
        void setColumnHorSizingEnabled(bool enabled);
        bool isGroupingEnabled() const;
        void setGroupingEnabled(bool enabled);
        GridLinesStyle gridLines() const;
        void setGridLines(GridLinesStyle lines);
        const QColor& gridLineColor() const;
        void setGridLineColor(const QColor& color);
        int gridLineWidth() const;
        void setGridLineWidth(int width);
        const QBrush& gridLineBrush() const;
        void setGridLineBrush(const QBrush& brush);
        bool alternatingRowColors() const;
        void setAlternatingRowColors(bool alternating);
        int columnHeight() const;
        void setColumnHeight(int height);
        int groupRowHeight() const;
        void setGroupRowHeight(int height);
        GridGroupRowStyle groupRowStyle() const;
        void setGroupRowStyle(GridGroupRowStyle style);
        int dataRowHeight() const;
        void setDataRowHeight(int height);
        const QPalette& cellPalette() const;
        void setCellPalette(const QPalette& palette);
        const QFont& cellFont() const;
        void setCellFont(const QFont& f);
        const QPalette& groupPalette(int depth);
        void setGroupPalette(int depth, const QPalette& palette);
        bool isFooterSummaryVisible() const;
        void setFooterSummaryVisible(bool visible);
        bool isGroupSummaryVisible() const;
        void setGroupSummaryVisible(bool visible);
        GridNewRowPlace newRowPlace() const;
        void setNewRowPlace(GridNewRowPlace place);
        const QString& newRowText() const;
        void setNewRowText(const QString& text);
        bool isMultiSelect() const;
        GridSelectionPolicy selectionPolicy() const;
        void setSelectionPolicy(GridSelectionPolicy policy);
        bool showFocusDecoration() const;
        void setshowFocusDecoration(bool value);
        bool rowAutoHeight() const;
        void setRowAutoHeight(bool autoHeight);
        RowHighlightEffect newRowHighlightEffect() const;
        void setNewRowHighlightEffect(RowHighlightEffect effect);
        int newRowFlashInterval() const;
        void setNewRowFlashInterval(int interval);
        double newRowOpacity() const;
        void setNewRowOpacity(double opcity);
        bool isMainMenuDisabled() const;
        void setMainMenuDisabled(bool disabled);
        bool isSummaryMenuDisabled() const;
        void setSummaryMenuDisabled(bool disabled);
        bool isFocusFollowsMouse() const;
        void setFocusFollowsMouse(bool enabled);
        bool fastScrollEffect() const;
        void setFastScrollEffect(bool value);
    private:
        bool m_columnAutoWidth;
        bool m_columnsHeader;
        bool m_columnsQuickCustomization;
        bool m_columnHeaderHints;
        int m_columnsDropDownCount;
        QColor m_fixedSeparatorColor;
        int m_fixedSeparatorWidth;
        bool m_isColumnHidingEnabled;
        bool m_isColumnMovingEnabled;
        bool m_isColumnHidingOnGroupingEnabled;
        bool m_isColumnHorSizingEnabled;
        bool m_isGroupingEnabled;
        GridLinesStyle m_gridLines;
        int m_gridLineWidth;
        QBrush m_gridLineBrush;
        bool m_alternatingRowColors;
        int m_columnHeight;
        int m_groupRowHeight;
        GridGroupRowStyle m_groupRowStyle;
        int m_dataRowHeight;
        QPalette m_cellPalette;
        QFont m_cellFont;
        QMap<int, QPalette> m_groupPalette;
        bool m_footerSummaryVisible;
        bool m_groupSummaryVisible;
        GridNewRowPlace m_newRowPlace;
        QString m_newRowText;
        GridSelectionPolicy m_selectionPolicy;
        bool m_showFocusDecoration;
        bool m_rowAutoHeight;
        RowHighlightEffect m_newRowHighlightEffect;
        int m_newRowFlashInterval;
        double m_newRowOpacity;
        bool m_mainMenuDisabled;
        bool m_summaryMenuDisabled;
        bool m_focusFollowsMouse;
        bool m_fastScrollEffect;
    };

    class GridTableVirtualRow
    {
    public:
        friend class GridVirtualTableEngine;
        friend class GridTableDataRowGraphic;
        friend class GridTableGroupSummaryGraphic;
        friend class GridTableGroupSummaryCellGraphic;
        enum VirtualRowType
        {
            VirtualRowUndefined,
            VirtualRowNewData,
            VirtualRowData,
            VirtualRowGroup,
            VirtualRowSummary
        };
	    GridTableVirtualRow();
        virtual ~GridTableVirtualRow();
        virtual void create(GridTableView* view);
        inline VirtualRowType type() const { return m_type; }
        void setRect(const QRect& rect, bool forceUpdate = false);
        const QRect& rect() const;
        GridTableView* view() const;
        void updateModelRow();
        void updateRect();
        void initialize(GridRow* row, VirtualRowType type);
        GridRow* gridRow() const;
        void registrColumnGraphic(const GridTableColumn* column);
        void unregistrColumnGraphic(const GridTableColumn* column);
        void updateGraphicFixedState(const GridTableColumn* column);
        void editorChanging(const GridTableColumn* column);
        void updateColumnLayout();
        void updateCellsData();
        void updateSummaryCellsData();
        bool focused() const;
        bool isRowSelected() const;
        bool isCellSelected(int cellIndex) const;
        bool isReady() const;
        void setReady(bool r);
        bool alternatingRowColors() const;
        GridTableCellGraphic* getCellGraphic(int cellIndex);
        void updateQuickRowButtonGraphic();
        void setOpacity(double value);
        double opacity() const;
        void updateRowMask(const QRect& maskRect);
        GridTableDataRowGraphic* dataRowGP() const;
        GridTableGroupRowGraphic* groupRowGP() const;
    protected:
        void createCells();
        void updateDataRowGraphic(bool visibility);
        void updateGroupSummaryGraphic(bool visibility);
        void createQuickRowButtonGraphic();
        void createDataRowGraphic();
        void createGroupGraphic();
        void createSummaryPanelGraphic();
        GridScrollableGraphicPrimitive* getRowParentGP() const;
    protected:
	    GridTableView* m_view;
    private:
        VirtualRowType m_type;
        GridTableQuickRowButtonGraphic* m_quickRowButtonGraphic;
        GridTableDataRowGraphic* m_dataRowGraphic;
        GridTableGroupRowGraphic* m_groupGraphic;
        GridTableGroupIndentGraphic* m_groupIndentGraphic;
        GridTableGroupSummaryGraphic* m_symmaryPanelGraphic;
        GridNewRowGraphic* m_newRowPanelGraphic;
        GridNewRowDelimeterGraphic* m_newRowDelimeterGraphic;
        GridRow* m_gridRow;
        int m_modelRowIndex;
        QRect m_rect;
        bool m_dataCellsVisibility;
        bool m_summaryCellsVisibility;
        bool m_ready;
        bool m_isNewRow;
        double m_opacity;
    };

    class GridVirtualTableEngine: public QObject
    {
        Q_OBJECT
    public:
        friend class GridTableView;
        friend class GridTableVirtualRow;
        GridVirtualTableEngine(GridTableView* view);
        virtual ~GridVirtualTableEngine();
        void createVirtualRows();
        void deleteVirtualRows();
        void beginUpdate(int topY, int width, int height, int offsetY, bool forceUpdate);
        void endUpdate();
        bool setNextModelRow(GridRow* modelRow);
        void registrColumnGraphic(const GridTableColumn* column);
        void unregistrColumnGraphic(const GridTableColumn* column);
        void updateGraphicFixedState(const GridTableColumn* column);
        void resetVirtualRows();
        void editorChanging(const GridTableColumn* column);
        void updateColumnLayout();
        void updateSummaryCellsData();
        GridRow* topRow() const;
        GridRow* bottomRow() const;
        GridTableVirtualRow* getVirtualRow(GridRow* modelRow, GridTableVirtualRow::VirtualRowType rowType);
        GridTableVirtualRow* getVirtualRowByPosY(int y);
        bool isRowVisible(GridRow* row);
        GridTableCellGraphic* getCellGraphic(GridRow* modelRow, int cellIndex);
        GridTableVirtualRow* newVirtualRow();
        void showNewRow();
        void hideNewRow();
        void setNewRowToEditState(GridRow* row);
        inline bool autoHeight() const { return m_autoHeightColumnRowCount > 0; }
        void setScrollX(int pos);
        void updateRowMask(const QRect& maskRect);
    protected slots:
        void updateNewRowOpacity();
    private:
        void saveEditingContext(GridTableVirtualRow* editingRow);
        bool setNextModelRowEx(GridRow* modelRow, bool summary);
        GridTableView* m_view;
        QVector<GridTableVirtualRow> m_virtualRows;
        QRect m_currentRowRect;
        int m_viewHeight;
        int m_defaultDataRowHeight;
        int m_defaultGroupRowHeight;
        int m_defaultSummaryHeight;
        int m_offsetY;
        QTimer* m_newRowAnimationTimer;
        int m_autoHeightColumnRowCount;
        QVector<int> m_autoHeights;

        struct WaitingRowParam
        {
            QRect rect;
            bool summary;
            GridRow* modelRow;
            GridTableVirtualRow* row;
        };
        typedef QVector<WaitingRowParam> WaitingRowParamList; 
        WaitingRowParamList m_waiting;
        int m_waitingIndex;
        GridRow* m_topRow;
        GridRow* m_bottomRow;
        GridTableVirtualRow* m_newVirtualRow;
        GridTableVirtualRow* m_transparentRow;
        double m_transparentRowOpacity;
        bool m_transparentRowAnimationOrder;
        bool m_forceUpdate;
    public:
        int get_model_row_height(GridRow* modelRow, int defaultDataHeight, int defaultGroupHeight,
            int defaultSummaryHeight, bool summary);
        int get_model_row_height_with_summary(GridRow* modelRow, int defaultDataHeight,
            int defaultGroupHeight, int defaultSummaryHeight, bool summaryVisible);

    };

    class GridTableQuickRowButtonGraphic: public GridBufferedGraphicPrimitive
    {
    public:
        GridTableQuickRowButtonGraphic(GridGraphicPrimitive* parentGraphic, GridTableVirtualRow* row);
    protected:
        virtual void paintEvent(QPainter& painter);
	    virtual void paintBufferEvent(QPainter& painter);
        virtual void mousePressEvent(const QPoint& point, QMouseEvent* e);
        virtual void mouseDoubleClickEvent(const QPoint& point, QMouseEvent* e);
    private:
	    GridTableVirtualRow* m_row;
    };

    class GridTableDataRowGraphic: public GridScrollableGraphicPrimitive
    {
        friend class GridVirtualTableEngine;
        friend class GridTableVirtualRow;
    public:
        GridTableDataRowGraphic(GridGraphicPrimitive* parent, GridTableVirtualRow* row);
        virtual ~GridTableDataRowGraphic();
        void updateCellsData();
        void updateBorder();
        void destroyCells();
        GridTableVirtualRow* row() const;
        void registrColumnGraphic(const GridTableColumn* column);
        void unregistrColumnGraphic(const GridTableColumn* column);
        void updateGraphicFixedState(const GridTableColumn* column);
        void editorChanging(const GridTableColumn* column);
    protected:
	    virtual void paintEvent(QPainter& painter);
        virtual void drawBorder2(QPainter& painter);
    private:
        GridTableVirtualRow* m_row;
        QHash<const GridTableColumn *, GridTableCellGraphic *> m_dataCells;
    };

    class GridTableGroupBranchButton: public GridGraphicPrimitive
    {
    public:
        friend class GridTableGroupRowGraphic;
        GridTableGroupBranchButton(GridTableGroupRowGraphic* parent, GridTableVirtualRow* row);
    protected:
        virtual void paintEvent(QPainter& painter);
        virtual void mousePressEvent(const QPoint& point, QMouseEvent* e);
        virtual void mouseOverEvent(const QPoint& pos);
        virtual void mouseOutEvent();
    private:
        GridTableVirtualRow* m_row;
        QColor m_bgcolor; //hack for MAC native style support
    };

    class GridTableGroupRowGraphic: public GridGraphicPrimitive
    {
    public:
        friend class GridTableVirtualRow;
        GridTableGroupRowGraphic(GridScrollableGraphicPrimitive* parentGraphic, GridTableVirtualRow* row);
        void updateBorder();
        GridRow* gridRow() const;
    protected:
        virtual void mousePressEvent(const QPoint& point, QMouseEvent* e);
        virtual void mouseDoubleClickEvent(const QPoint& point, QMouseEvent* e);
        virtual void mouseMoveEvent(const QPoint& point, QMouseEvent* e);
        virtual void mouseOverEvent(const QPoint& pos);
        virtual void mouseOutEvent();
        virtual void drawBorder(QPainter& painter);
        virtual void paintEvent(QPainter& painter);
        virtual void resizeEvent(const QRect& rect);
        virtual bool startDragDropEvent(const QPoint& p);
        virtual GridDragDropObject* createDragDropObject(const QPoint& point);
    protected:
        QString getPaintText() const;
    private:
        GridTableVirtualRow* m_tableRow;
        GridTableGroupBranchButton* m_branchButon;
    };

    class GridTableGroupIndentGraphic: public GridGraphicPrimitive
    {
    public:
        GridTableGroupIndentGraphic(GridScrollableGraphicPrimitive* parentGraphic, GridTableVirtualRow* row);
    protected:
        virtual void paintEvent(QPainter& painter);
    private:
        GridTableVirtualRow* m_row;
    };

    class GridTableSummaryGraphicBase: public GridScrollableGraphicPrimitive
    {
    public:
        GridTableSummaryGraphicBase(GridGraphicPrimitive* parent, GridTableView* view);
        virtual ~GridTableSummaryGraphicBase();
        void updateBorder();
        void registrColumnGraphic(const GridTableColumn* column);
        void unregistrColumnGraphic(const GridTableColumn* column);
        void updateGraphicFixedState(const GridTableColumn* column);
        void destroySummaryCells();
        void updateSummaryCellsData();
        void updateCellsLayout();
        void setMask(const QRect& maskRect);
        void updateMask(int leftIndent);
    protected:
        virtual GridTableSummaryCellGraphic* createSummaryCell(const GridTableColumn* column) = 0;
        virtual int getCellLeftIndent() = 0;
        GridTableView* m_view;
        QRect m_origMask;
    private:
        QHash<const GridTableColumn *, GridTableSummaryCellGraphic *> m_summaryCells;
    };

    class GridTableFooterSummaryGraphic: public GridTableSummaryGraphicBase
    {
    public:
        GridTableFooterSummaryGraphic(GridGraphicPrimitive* parent, GridTableView* view);
    protected:
        virtual GridTableSummaryCellGraphic* createSummaryCell(const GridTableColumn* column);
        virtual int getCellLeftIndent();
    private:
    };

    class GridTableGroupSummaryGraphic: public GridTableSummaryGraphicBase
    {
        friend class GridTableVirtualRow;
    public:
        GridTableGroupSummaryGraphic(GridGraphicPrimitive* parent, GridTableVirtualRow* row);
    protected:
        virtual void paintEvent(QPainter& painter);
        virtual GridTableSummaryCellGraphic* createSummaryCell(const GridTableColumn* column);
        virtual int getCellLeftIndent();
    private:
        GridTableVirtualRow* m_row;
    };

    class GridTableSummaryCellGraphic: public GridGraphicPrimitive
    {
    public:
        GridTableSummaryCellGraphic(GridTableView* view, GridGraphicPrimitive* parent);
        void setDataBinding(ModelDataBinding* dataBinding);
        ModelDataBinding* dataBinding() const;
        void setDirtyData();
        virtual void setSummaryType( SummaryType type) = 0;
    protected:
        virtual void paintEvent(QPainter& painter);
        virtual void showContextMenuEvent(const QPoint& pos, QContextMenuEvent* event);
        virtual const SummaryValue& getValue();
        virtual const QString& getSummaryDisplayText();
        virtual void loadData() = 0;
        GridTableView* m_view;
        SummaryValue m_value;
        QString m_displayText;
    private:
        ModelDataBinding* m_dataBinding;
        bool m_dirtyData;
    };

    class GridTableGroupSummaryCellGraphic: public GridTableSummaryCellGraphic
    {
    public:
        GridTableGroupSummaryCellGraphic(GridGraphicPrimitive* parent, GridTableVirtualRow* tableRow);
        virtual void setSummaryType( SummaryType type);
    protected:
        virtual void loadData();
    private:
        GridTableVirtualRow* m_tableRow;
    };

    class GridTableFooterSummaryCellGraphic: public GridTableSummaryCellGraphic
    {
    public:
        GridTableFooterSummaryCellGraphic(GridGraphicPrimitive* parent,
                                          GridTableView* view,
                                          ModelDataBinding* dataBinding);
        virtual void setSummaryType( SummaryType type);
    protected:
        virtual void loadData();
    private:
    };

    class GridTableCellGraphic: public GridEmbedWidgetGraphic, public GridEditorSite
    {
        Q_OBJECT
    public:
        friend class GridTableDataRowGraphic;
        friend class GridTableView;
        GridTableCellGraphic(GridTableView* view, GridGraphicPrimitive* parent,
            GridTableVirtualRow* tableRow, 
            ModelDataBinding* dataBinding, int columnIndex);
        virtual ~GridTableCellGraphic();
        void loadData();
        void setDirtyData(bool flag);
        bool isDirtyData() const { return m_dirtyData; }
        void updateBorder();
        GridEditor* editorItem();
        void resetEditItem();
        GridRow* gridRow() const;
        void initEditorOption(GridEditorStyleOption& option);
    protected:
        virtual void resizeEvent(const QRect& rect);
        virtual void paintEvent(QPainter& painter);
        virtual void keyPressEvent(QKeyEvent* e);
        virtual void mouseMoveEvent(const QPoint& point, QMouseEvent* e);
        virtual void mousePressEvent(const QPoint& point, QMouseEvent* e);
        virtual void mouseReleaseEvent(const QPoint& point, QMouseEvent* e);
        virtual void mouseDoubleClickEvent(const QPoint& point, QMouseEvent* e);
        virtual void windowBlockedEvent(QEvent* e);
        virtual void windowUnBlockedEvent(QEvent* e);
        virtual void mouseOverEvent(const QPoint& pos);
        virtual void mouseOutEvent();
        virtual bool startDragDropEvent(const QPoint& p);
        virtual GridDragDropObject* createDragDropObject(const QPoint& point);
        void beginUpdateByMousePress();
        void endUpdateByMousePress();
    protected:
        virtual void drawBorder(QPainter& painter);
    protected:
        virtual void repaint();
        virtual void update(bool fullyViewUpdate = false);
        virtual void update(const QRect& rect);
        virtual QPoint globalPos() const;
        virtual ModelDataBinding* dataBinding() const;
    public:
        virtual int columnIndex() const;
    private:
        GridTableView* m_view;
        ModelDataBinding* m_dataBinding;
        GridTableVirtualRow* m_tableRow;
        bool m_dirtyData;
        bool m_effectLoadData;
        int m_columnIndex;
        GridEditor* m_editor;
        int m_pressedRowIndex;
        bool m_focusByMousePress;
        bool m_stopRowEditByMousePress;
        int m_viewUpdateByMousePressCount;
        EffectSettings m_effSettings;
    };

    bool do_stretch_columns(GridTableView* view, const GridViewBase::GridColumns& cols, int visualIndex, int width);

    class GridTableFooterGraphic: public GridFooterPanelGraphic
    {
        friend class GridTableView;
        friend class GridTableViewGraphic;
    public:
        GridTableFooterGraphic(GridTableView* view, const QRect& rect);
        virtual ~GridTableFooterGraphic();
        virtual void updateBorder();
        virtual void updateSummaryLayout();
        GridTableFooterSummaryGraphic* summaryGraphic() const;
        static void paintSummaryBackgroud(QPainter& painter, const QPalette& palette, 
            const QRect& rect, QWidget* site);
    protected:
        virtual void paintEvent(QPainter& painter);
    private:
        GridTableFooterSummaryGraphic* m_summaryGraphic;
    };

    class GridNewRowGraphic: public GridGraphicPrimitive
    {
    public:
        GridNewRowGraphic(GridTableView* view);
    protected:
        virtual void paintEvent(QPainter& painter);
        virtual void mouseReleaseEvent(const QPoint& point, QMouseEvent* e);
    private:
        GridTableView* m_view;
    };

    class GridNewRowDelimeterGraphic: public GridGraphicPrimitive
    {
    public:
        GridNewRowDelimeterGraphic(GridTableView* view);
    protected:
        virtual void paintEvent(QPainter& painter);
    };

    class QTITAN_EXPORT GridContextMenu: public QMenu
    {
    public:
        GridContextMenu(QWidget *parent);
        void show(int gx, int gy, GridGraphicPrimitive* hit);
        GridGraphicPrimitive* hit() const;
    private:
        GridGraphicPrimitive* m_hit;
    };

    class QTITAN_EXPORT GridSummaryContextMenu: public QMenu
    {
    public:
        GridSummaryContextMenu(QWidget *parent);
    };

    class QTITAN_EXPORT GridTableColumnEditor: public QObject
    {
    public:
        GridTableColumnEditor();
        virtual ~GridTableColumnEditor();
        virtual QWidget* createEditorWidget(GridTableColumnBase* column) = 0;
    };

    class GridGraphicSitePublic: public GridGraphicSite
    {
        GridGraphicSitePublic() : GridGraphicSite(0) {}
        friend class Qtitan::GridTableView;
        friend class Qtitan::GridColumnChooseWidget;
        friend class Qtitan::GridTableViewGraphic;
        friend class Qtitan::GridVirtualTableEngine;
        friend class Qtitan::GridTableDragColumnObject;
        friend class Qtitan::GridTableSelectionObject;
        friend class Qtitan::GridTableColumnGraphic;
        friend class Qtitan::GridColumnEditorGraphic;
        friend class Qtitan::GridTableCellGraphic;
    };

    extern void get_cell_border_params(GridTableView* view, int& borderRight, int& borderBottom, QBrush& borderBrush);
}; //namespace Qtitan


#endif //QTN_GRIDTABLEVIEW_H
